/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.viewlets.ui.FavoritesViewlet");

dojo.require("dojo.i18n");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.viewlets.ui.Viewlet");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.page.FavoritesController");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");

dojo.requireLocalization("com.ibm.rational.cm.web.viewlets.ui", "ViewletMessages");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function(){

var messages, dashboardMessages;
var cq = com.ibm.rational.cm.web;
var loginController = com.ibm.rational.cm.cqLoginController;
var favController = com.ibm.rational.cm.favoritesController;

dojo.declare(
	"com.ibm.rational.cm.web.viewlets.ui.FavoritesViewlet",
	com.ibm.rational.cm.web.viewlets.ui.Viewlet,
{
	init: function(){
		messages = this.messages;
		dashboardMessages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

		// set viewlet title (change it from viewlet chooser)
		this.getSite().setTitle(messages.FavoritesViewletTitle);

		var container = this.listContainer;
		if(!container){
			container = this.listContainer = dojo.doc.createElement("UL");
			container.style.margin = "0px";
			container.style.paddingTop = "5px";
			container.style.paddingLeft = "5px";
			this.mainContent.appendChild(container);
		}
		this._items = [];
		var self = this;
		var subscribe = dojo.subscribe;
		this._subscriptions = [
			subscribe("CQ_LOGIN_SUCCESS", function(){
				self.populateFavorites(true);
			}),
			subscribe("CQ_FAVORITE_INIT", function(){
				// no need to forceLoad since it is guarantied that favorites are already loaded
				self.populateFavorites(false);
			}),
			subscribe("CQ_FAVORITE_ADDED", function(message){
				self._items.push(message);
				self.update();
			}),
			subscribe("CQ_FAVORITE_REMOVED", function(message){
				for(var i = self._items.length-1; i >= 0; i--){
					if(self._items[i].id == message.id){
						self._items.splice(i, 1);
					}
				}
				self.update();
			})
		];
	},
	
	/*
	 * Called on init as well as "Refresh"
	 */
	refresh: function(){
		// populate favorites if already authenticated
		var self = this;
		loginController.process({
			callback: function(){
				self.populateFavorites(true); // need to forceLoad for both init & refresh
			},
			onCancel: function(){
				self.showMainContent(false);
			},
			noPrompt: true
		});
	},
	
	/*
	 * Override com.ibm.rational.cm.web.viewlets.ui.Viewlet
	 */
	load: function(){
		this.populateFavorites(true);
	},
	
	populateFavorites: function(/* boolean */ forceLoad){
		var self = this;
		var site = this.getSite();
		favController.loadFavorites({
			onBegin: function(){
				site.setLoading(true);
				self.showMainContent(true);
			},
			onLoad: function(/* Array */ items){
				site.setLoading(false);
				self._items = items;
				self.update();
				site.doneRefresh();
			},
			onCancel: function(){
				site.setLoading(false);
				self.showMainContent(false);
				site.doneRefresh();
			},
			forceLoad: forceLoad
		});
	},
	
	update: function(){
		// updates favorites list
		var container = this.listContainer;
		if(this._items.length > 0){
			container.innerHTML = "";
			dojo.forEach(this._items, function(item){
				var li = dojo.doc.createElement("li");
				li.style.listStyleType = "none";
				var a = dojo.doc.createElement("A");
				a.href = cq.getActionHref(item.type, item.id, item.label, true);

				// add icon
				var icon = dojo.doc.createElement("IMG");
				icon.style.width = "16px";
				icon.style.height = "16px";
				icon.src = cq.getIconUrl(item.type);
				a.appendChild(icon);

				a.appendChild(dojo.doc.createTextNode(" "+item.label));
				li.appendChild(a);
				container.appendChild(li);
			});
		}else{
			container.innerHTML = '<li style="list-style-type:none;">'+dashboardMessages.pagedTableNoResults+'</li>';
		}
		
		// updates Add/Delete links when editing
		if(this.isEditing()){
			if(!this._addLink){
				this._site.getActionArea().innerHTML = "";
				var addDiv = dojo.doc.createElement("div");
				addDiv.className = "action add";
				var addLink = this._addLink = dojo.doc.createElement("a");
				addLink.innerHTML = messages.Add;
				addLink.href = "";
				addDiv.appendChild(addLink);
			
				this.connect(addLink, "onclick", this._openNavigatorDialog);
				this._site.getActionArea().appendChild(addDiv);
			}
		}
	},
	
	_openNavigatorDialog: function(e){
		dojo.stopEvent(e);
		
//		dojo["require"]("com.ibm.rational.cm.web.ui.utils.NavigatorChooser");
		com.ibm.rational.cm.openNavigatorChooser({
			filter: {
				type: [ "LIST", "CHART", "REPORT" ] // don't show REPORT-FORMAT
			},
			multiSelect: true,
			onOk: function(arr, store){
				dojo.forEach(arr, function(item){
					favController.addFavorite({
						type: store.getValue(item, "type"),
						stableLocation: store.getIdentity(item),
						title: store.getValue(item, "title")
					});
				});
			}
		});
	},
	
	destroy: function(){
		dojo.forEach(this._subscriptions, dojo.unsubscribe);
		this.inherited(arguments);
	}
});

})();