/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.viewlets.ui.QueryViewlet");

dojo.require("com.ibm.rational.cm.web.viewlets.ui.PagedTableViewlet");

dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.viewlets.ui.utils.QueryPreferenceProvider");
dojo.require("com.ibm.rational.cm.web.viewlets.ui.utils.PresentationFieldsPreferenceProvider");

(function(){

var cq = com.ibm.rational.cm.web;
var loginController = com.ibm.rational.cm.cqLoginController;
var PREF_ID_QUERY = "query";
var PREF_ID_FIELDS = "presentationFields";
var PREF_ROWS = "rowsPerPage";
var BASE_URL = "cqqueryresults.cq";
var LOGIN_ICON_URL = dojo.moduleUrl("com.ibm.rational.cm.web.viewlets", "graphics/circle_arrow.gif");

dojo.declare(
	"com.ibm.rational.cm.web.viewlets.ui.QueryViewlet",
	com.ibm.rational.cm.web.viewlets.ui.PagedTableViewlet,
{
	contentClass: "com-ibm-rcm-QueryViewlet",
	
	constructor: function(){
		this._preferenceProviders = {};
		this._mastershipField = null;
		this._entries = [];
		this._details = [];
		this._detailsWidgets = {};
		
		// subscribe to login success event so that query result is populated when login is done somewhere else ("ClearQuest Records" tab)
		var self = this;
		this._subscriptions = [
			dojo.subscribe("CQ_LOGIN_SUCCESS", function(){
				self.populateResults(false); // don't prompt for login since login is guaranteed
			})
		];
	},

	/*
	 * Overrides PagedTableViewlet
	 * so that viewlet loads startup query info on initialization
	 */
	init: function(){
		this.inherited(arguments);
		
		// If query has not been specified, set viewlet title (change it from label in viewlet chooser)
		var resourceId = this.getPreference(PREF_ID_QUERY);
		if(resourceId == "-1" || !resourceId || resourceId.length === 0){
			this.getSite().setTitle(this.messages.QueryViewletTitle);
		}
		
		this._renderNoResults();
		this.populateResults(false);
	},
	
	populateResults: function(/* boolean */ doPrompt){
		this._refresh = true;
		this.fetch(null, 0, this.getItemsPerPage(), false, false, doPrompt);
		this._refresh = false;
	},
	
	refresh: function(){
		this._refresh = true;
		delete this._totalResults;
		this.inherited(arguments);
		this._refresh = false;
	},

	clearRows: function(){
		if(this._detailsWidgets){
			for(var key in this._detailsWidgets){
				this._detailsWidgets[key].destroy();
			}
		}
		this._detailsWidgets = {};
	},

	/*
	 * Override PagedTableViewlet
	 */
	getItemsPerPage: function(){
		return this.getPreference(PREF_ROWS);
	},
	
	/*
	 * Overrides PagedTableViewlet and called from onInit & refresh
	 * Passed ServiceResponseHandler is ignored since we use our own XHR handler
	 * Added doPrompt argument to original method in PagedTableViewlet
	 */
	fetch: function(/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details, /* boolean */ doPrompt){
		var resourceId = this.getPreference(PREF_ID_QUERY);
		this._setLoading(false);
		var site = this.getSite();
		site.setLoading(false);
		if(resourceId == "-1" || !resourceId || resourceId.length === 0){
			this._renderNoResults();
			site.doneRefresh();
		}else{
			this.getPreferenceProvider(PREF_ID_QUERY).initDefault(resourceId);
			var self = this;
			loginController.process({
				callback: function(){
					self.executeQuery(resourceId, startIndex, count);
				},
				onCancel: function(){
					self._renderNoResults();
					site.doneRefresh();
				},
				noPrompt: !doPrompt // doPrompt is undefined when called from refresh, and false for onInit
			});
		}
	},
	
	/*
	 * Override PagedTableViewlet
	 * show different messages depending on conditions
	 */
	getNoResultsMessage: function(){
		var resourceId = this.getPreference(PREF_ID_QUERY);
		if(resourceId == "-1" || !resourceId || resourceId.length === 0){ // no query specified
			return this.getNoQueryMessage();
		}else if(!loginController.isAuthenticated()){ // not logged in
			return this.getNotLoggedInMessage();
		}else{ // no records
			return this.getNoHitsMessage();
		}
	},
	
	getNotLoggedInMessage: function(){
		var _needLogin = this._needLogin;
		if(!_needLogin){
			var doc = dojo.doc;
			_needLogin = this._needLogin = doc.createElement("DIV");
			
			var msgDiv = doc.createElement("DIV");
			msgDiv.innerHTML = this.messages.NeedLogin;
			_needLogin.appendChild(msgDiv);

			var linkDiv = doc.createElement("DIV");
			var a = doc.createElement("A");
			var self = this;
			dojo.attr(a, {
				href: "javascript:void(0);",
				innerHTML: '<span class="dijitInline cqLoginIcon"></span> '+this.messages.Login,
				onclick: function(){
					self.populateResults(true);
				}
			});
			linkDiv.appendChild(a);

			_needLogin.appendChild(linkDiv);
		}
		this.getSite().setSecondaryTitle("("+this.messages.NotLoggedIn+")");
		return _needLogin;
	},
	
	/*
	 * Executes the query of a given resourceId (stableLocation)
	 */
	executeQuery: function(/* String */ resourceId, /* int */ startIndex, /* int */ count){
		var site = this.getSite();
		var self = this;
		var setLoading = function(/* boolean */ loading){
			self._setLoading(loading);
			site.setLoading(loading);
		}
		var content = {
			action: "ExecuteQuery",
			resourceId: resourceId,
			startIndex: (startIndex+1),
			rowCount: count,
			format: "JSON"
		}
		// pass refresh=true for first query execution and refresh, not for pagination
		if(this._refresh){
			content.refresh = true;
		}
		cq.xhrGet({
			url: BASE_URL,
			content: content,
			onBegin: function(){
				setLoading(true);
			},
			load: function(response, ioArgs){
				// error handling
				if(!response || !response.resultSetData){
					var errMsg = (response && response.DetailedMessage) ? response.DetailedMessage : cq.getCommonString().errMsg;
					self._totalResults = -1;
					self._showMessage(dojo.doc.createTextNode(errMsg));
					setLoading(false);
					site.doneRefresh();
					site.setSecondaryTitle("");
					return;
				}

				var resultSetData = response.resultSetData;
				dojo.forEach(resultSetData.colData, function(item){
					if(item.isHiddenMastership){
						self._mastershipField = item.field;
					}
					// In Patch 2009C or above, item.field will be field path, and item.name will be display name
					// In older versions (2009A or 2009B), there will be no item.name, and only display name will be given (item.field)
					if(item.name){
						self._hasFieldPath = true;
					}
				});

				dojo.forEach(resultSetData.rowData, function(row, i){
					self._entries[startIndex + i] = row;
					self._details[startIndex + i] = row;
				});
				var totalResults = resultSetData.totalNumOfRows;
				self._totalResults = (totalResults >= 0) ? totalResults : -1;
				self._totalResultsChanged();

				var propName = (self._hasFieldPath) ? "fieldPath" : "displayName";
				var setFields = function(obj){
					var colData = obj.colData;
					// In Patch 2009C or above, item.field will be field path, and item.name will be display name
					// In older versions (2009A or 2009B), there will be no item.name, and only display name will be given (item.field)
					self._fields = dojo.map(obj.value, function(item){
						return colData[item][propName];
					});
					self._render();
				}
						
				// set display fields
				var prefObj = self._prefObj;
				if(!prefObj){ // get presentation fields from preferences
					var prefValue = self.getPreference(PREF_ID_FIELDS);
					if(prefValue){
						prefObj = self._prefObj = dojo.fromJson(prefValue);
					}
				}
				if(prefObj && prefObj != -1){
					 setFields(prefObj)
				}else{
					self.getPreferenceProvider(PREF_ID_FIELDS).getColumnData(resourceId, function(prefObj){
						self._prefObj = prefObj;
						setFields(prefObj);
					});
				}
			},
			handle: function(response, ioArgs){
				setLoading(false);
				site.doneRefresh();
			},
			onCancel: function(){
				self._totalResults = -1;
				self._renderNoResults();
				setLoading(false);
				site.doneRefresh();
			},
			handleError: true,
			preventCache: true
		});		
	},
	
	/*
	 * Override PagedTableViewlet
	 * need to override, otherwise "fetch" won't be called when saving preferences
	 */
	getResultCount: function(){
		var resourceId = this.getPreference(PREF_ID_QUERY);
		if(resourceId == "-1" || !resourceId || resourceId.length === 0){
			return -1;
		}else{
			return (this._totalResults >= 0) ? this._totalResults : -1;
		}
	},
	
	/*
	 * Overrides PagedTableViewlet
	 * returns an array of TD elements
	 */
	renderEntry: function(/* object */ entryData, /* object */ detailsData, /* boolean */ zoomed){
		// create icon cell
		var mastershipField = this._mastershipField;
		var iconCell = dojo.doc.createElement("TD");
		dojo.addClass(iconCell, "iconCell");
		var remote = (mastershipField) ? (entryData[mastershipField] != true) : false;
		iconCell.innerHTML = '<div class="iconContainer"><img src="'+cq.getIconUrl("RECORD", remote)+'" /></div>';

		// create title cell
 		var titleCell = dojo.doc.createElement("td");
//		dojo.addClass(titleCell, "titleCell");
		var titleWrapper = dojo.doc.createElement("div");
		dojo.addClass(titleWrapper, "titleWrapper");
		var str = "";
		var titleStr = "";
		dojo.forEach(this._fields, function(sel, idx){
			if(idx > 0){
				str += ": ";
			}
			str += cq.getAbbreviatedString(entryData[sel], 60);
			titleStr = entryData[sel];
		});
		var titleLink = dojo.doc.createElement("a");
		titleLink.href = cq.getActionHref("RECORD", entryData.recordId, titleStr, true);
		// truncate strings longer than 100 characters
		titleLink.innerHTML = str;
		titleWrapper.appendChild(titleLink);
		titleCell.appendChild(titleWrapper);

		// add hover popup
		// TODO : don't we have to destroy jazz.ui.HoverPopup widget somewhere??
		var self = this;
		new jazz.ui.HoverPopup({
			around: titleLink,
			createContents: function(hp){
				var node = dojo.doc.createElement("div");
				dojo.addClass(node, "com-ibm-rcm-ResultSetDetailsPopup");
				node.appendChild(self._getDetailsWidget(entryData).domNode);
				return node;
			},
			isEnabled: function(hp){
				return !self.isZoomed();
			}
		});

		return [ iconCell, titleCell ];
	},
	
	/*
	 * Override PagedTableViewlet
	 * details is empty for now
	 */
	renderDetails: function(/* object */ detailsData){
		var emptyCell = dojo.doc.createElement("TD");
		var detailsCell = dojo.doc.createElement("TD");
		dojo.addClass(detailsCell, "detailsCell");
		detailsCell.appendChild(this._getDetailsWidget(detailsData).domNode);
		return [ dojo.doc.createElement("TD"), detailsCell ];
	},

	_getDetailsWidget: function(entry){
		var recordId = entry.recordId;
		var detailsWidget = this._detailsWidgets[recordId];
		if(!detailsWidget){
			// create details widget
			detailsWidget = this._detailsWidgets[recordId] = new com.ibm.rational.cm.web.viewlets.ui.ResultSetDetails({
				messages: this.messages,
				colData: this._prefObj.colData,
				data: entry,
				_hasFieldPath: this._hasFieldPath
			});
			detailsWidget.populate();
		}
		return detailsWidget;
	},

	getPreferenceProvider: function(preferenceId){
		if(!this._preferenceProviders[preferenceId]){
			if(preferenceId == PREF_ID_QUERY){
//				dojo["require"]("com.ibm.rational.cm.web.viewlets.ui.utils.QueryPreferenceProvider");
				this._preferenceProviders[preferenceId] = new com.ibm.rational.cm.web.viewlets.ui.utils.QueryPreferenceProvider({
					prefId: preferenceId,
					fieldsPrefId: PREF_ID_FIELDS
				});
			}else if(preferenceId == PREF_ID_FIELDS){
//				dojo["require"]("com.ibm.rational.cm.web.viewlets.ui.utils.PresentationFieldsPreferenceProvider");
				this._preferenceProviders[preferenceId] = new com.ibm.rational.cm.web.viewlets.ui.utils.PresentationFieldsPreferenceProvider({
					prefId: preferenceId
				});
			}
		}	
		return this._preferenceProviders[preferenceId];
	},
	
	/*
	 * This will get called after preference has been saved (not preference value changes)
	 */
	settingsChanged: function(oldSettings, newSettings){
		var updateTitle = oldSettings.userTitle == true && newSettings.userTitle == false;
		var site = this.getSite();
		var oldPref = oldSettings.preferences;
		var newPref = newSettings.preferences;
		var queryChanged = (oldPref[PREF_ID_QUERY] != newPref[PREF_ID_QUERY]);
		var fieldsChanged = (oldPref[PREF_ID_FIELDS] != newPref[PREF_ID_FIELDS]);
		if(queryChanged || fieldsChanged){
			if(fieldsChanged){
				this._prefObj = dojo.fromJson(newPref[PREF_ID_FIELDS]);
			}
			this.reset(true);
			site.refresh();
			if(queryChanged){
				updateTitle = true;
			}
		}else if(oldPref[PREF_ROWS] != newPref[PREF_ROWS]){
			this.reset(false);
			this.update();
		}
		
		// getTextLabel is not CustomPreferenceProvider API
		// but is provided by the RequestsQueryPreferenceProvider
		if(updateTitle){
			site.setTitle(this.getPreferenceProvider(PREF_ID_QUERY).getTextLabel(newPref[PREF_ID_QUERY]), true);
		}
	},
	
	destroy: function(){
		dojo.forEach(this._subscriptions, dojo.unsubscribe);
		this.inherited(arguments);
	}
});

/*
 * Widget that is shown in zoomed state or hover popup
 */
dojo.declare(
	"com.ibm.rational.cm.web.viewlets.ui.ResultSetDetails",
	[ dijit._Widget, dijit._Templated ],
{
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web.viewlets", "ui/templates/ResultSetDetails.html"),
	
	colData: null,
	data: null,
	_hasFieldPath: false,
	
	setColumnData: function(colData){
		this.colData = colData;
	},
	
	setData: function(data){
		this.data = data;
	},
	
	populate: function(){
		// In Patch 2009C or above, item.field will be field path, and item.name will be display name
		// In older versions (2009A or 2009B), there will be no item.name, and only display name will be given (item.field)
		var propName = (this._hasFieldPath) ? "fieldPath" : "displayName";
		dojo.forEach(this.colData, function(col, idx){
			var par = (idx % 2 == 0) ? this._columnOne : this._columnTwo;
			var detailsRow = dojo.doc.createElement("DIV");
			dojo.addClass(detailsRow, "detailsRow");
			
			detailsRow.innerHTML = '<div class="detailsLabelCell">'+col.displayName+': </div><div class="detailsValueCell">'+this.data[col[propName]]+'</div>';
			par.appendChild(detailsRow);
		}, this);
	}
});
	
})();