/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletChooserTree");

/*
 * Imports 
 */
dojo.require("jazz.ui.ModeledTree");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ImageIconItem");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function() {

/*
 * Aliases
 */
var ModeledTree = jazz.ui.ModeledTree;
var ImageIconItem = com.ibm.team.dashboard.web.ui.internal.ImageIconItem;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var SERVICE_IMG = "com.ibm.team.dashboard.common.internal.service.IDashboardImageService";

/*
 * Class 
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletChooserTree",
	ModeledTree, {

	/*
	 * Custom double-click handler
	 */
	onDblClickHandler: null,

	/*
	 * Static Variable Container 
	 */
	_static: function() {}, 	

	/*
 	 * Public Method
 	 * 
 	 * @param Object args
 	 * 
 	 * @return ViewletChooserTree
 	 * @overrides super
 	 */	
	constructor: function(args) {
		this._dataServerRoot = args.dataServerRoot;
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		this._initializeStaticConstants();
		this.inherited(arguments);
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticConstants: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}

		this._static._instanceCount = 1;

		this._static._TYPE_CATEGORY = "c";
		this._static._TYPE_VIEWLET = "v";
		
		this._static._DEFAULT_CATEGORY_ICON = 
			dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/icons/folder.gif");

	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return ImageIconItem
 	 * @overrides super
 	 */
	createTreeItem: function(modelObj) {
		
		if (modelObj.type === this._static._TYPE_CATEGORY) {
			this._addCategoryImageData(modelObj);
		} else {
			this._addViewletImageData(modelObj);
		}

		return new ImageIconItem(modelObj);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */		
	_addCategoryImageData: function(modelObj) {
		modelObj.iconSrc = this._static._DEFAULT_CATEGORY_ICON;
		modelObj.labelClass = "category-label";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object modelObj
 	 * 
 	 * @return void
 	 */
	_addViewletImageData: function(modelObj) {
		var params = {definitionId: modelObj.viewletDefId};
		if (modelObj.icon) {
			if (modelObj.icon.indexOf("://") === -1) {
				// Icon format:
				// com.ibm.team.dashboard.viewlets.service/ui/internal/images/icon-status.gif
				params.path = modelObj.icon;
			}
			else {
				// Icon format:
				// https://jazz.net/favicon.ico
				params.url = modelObj.icon;
			}
		}
		modelObj.iconSrc = Util.getServiceURL(SERVICE_IMG, null, params, this._dataServerRoot);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object e
 	 * @param Object item
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	_doSelection: function(e, item) {
		this.inherited(arguments);
		this.onClick();
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	onClick: function() {
		// NO-OP
	},

	/*
 	 * Public Method
 	 * 
 	 * @param jazz.ui.tree.AbstractItem item
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	onDblClick: function(item) {
		// Call super (for non-leaf nodes)
		this.inherited(arguments);
		// Call custom double-click handler
		if (this.onDblClickHandler) {
			this.onDblClickHandler();
		}
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		this.inherited(arguments);
	}
	
});

})();
	
	