/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProjectAreaEditorPage"); //$NON-NLS-1$
/*
 * Imports
 */
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
(function(){

	/*
	 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
	 * future with minimal or no warning.
	 * 
	 * Base class for all contributed project area editor pages.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.AbstractProjectAreaEditorPage", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		// Properties //----------------------------------------------------------->
		
		/*
		 * The project area item UUID
		 */
		itemUUID: "", //$NON-NLS-1$
		/*
		 * A reference to this tab's container
		 */
		tabContainer: null,
		
		/*
		 * The ID of this tab
		 */
		tabID: "", //$NON-NLS-1$
		/*
		 * The title of the tab
		 */
		tabTitle: "", //$NON-NLS-1$
		// Public Methods //------------------------------------------------------->
		
		/*
		 * Initializes this page.
		 *
		 * Called by this page's container.
		 *
		 * @param void
		 *
		 * @return void
		 * @overrides super
		 */
		postCreate: function(){
		},
		
		/*
		 * Notifies this page's container to check for pending changes and update the dirty state.
		 *
		 * @param void
		 *
		 * @return void
		 */
		setDirty: function(){
			this.tabContainer.setDirty(this.tabContainer.changesPending());
		},
		
		/*
		 * Saves the page's input and mark this page as no longer containing pending changes.
		 * If an error occurs, this page should continue to indicate that it has pending changes
		 * to provide the user the opportunity to correct any problems and attempt to save again.
		 *
		 * Called by this page's container.
		 *
		 * @param Function callback - A callback used to notify the page container
		 * 							  whether the save operation succeeded or failed.
		 *
		 * 		Callback method signature:
		 *
		 * 				@param Error errorObject - An optional error object
		 * 										   specified only if the save
		 * 										   operation failed.
		 *
		 * 				@return void
		 *
		 * 				callback(errorObject)
		 *
		 * 		e.g. Save operation succeeded
		 * 		callback();
		 *
		 * 		e.g. Save operation failed
		 * 		callback(new Error("Failed to save editor page: " +
		 * 			this.pageTitle));
		 *
		 * @return void
		 */
		doSave: function(callback){
			callback();
		},
		
		/*
		 * Refreshes this page's content.
		 *
		 * Called by this page's container.
		 *
		 * @param void
		 *
		 * @return void
		 */
		refresh: function(){
		},
		
		/*
		 * Called by the tab container any time the tab is selected to
		 * be the active tab.
		 *
		 * @param void
		 *
		 * @return void
		 */
		activateTab: function(){
		
		},
		
		/*
		 * Indicates if this page has pending changes that need to
		 * be saved.
		 *
		 * @param void
		 *
		 * @return boolean true if the tab has changes pending
		 */
		changesPending: function(){
			return false;
		},
		
		/*
		 * Indicates if this page is in a state where it is able to be
		 * saved.
		 *
		 * @param void
		 *
		 * @return boolean true if the tab is in a state where it can be saved
		 */
		canSave: function(){
			return true;
		},
		
		/*
		 * Used to display a pending message in the shared message area of the page container.
		 */
		displayPendingMessage: function(message){
			this.tabContainer.displayPendingMessage(message);
		},
		
		/*
		 * Used to display a warning message in the shared message area of the page container.
		 *
		 * @param String the warning message to display
		 *
		 * @return void
		 */
		displayWarningMessage: function(message){
			this.tabContainer.displayWarningMessage(message);
		},
		
		/*
		 * Used to display an error message in the shared message area of the page container.
		 *
		 * @param String the error message to display
		 *
		 * @return void
		 */
		displayErrorMessage: function(message){
			this.tabContainer.displayErrorMessage(message);
		},
		
		/*
		 * Used to display a success message in the shared message area of the page container.
		 *
		 * @param String the success message to display
		 *
		 * @return void
		 */
		displaySuccessMessage: function(message){
			this.tabContainer.displaySuccessMessage(message);
		},
		
		/*
		 * Used to clear the message in the shared message area of the page container.
		 *
		 * @param void
		 *
		 * @return void
		 */
		clearMessage: function(){
			this.tabContainer.clearMessage();
		},
		
		/*
		 * Destroys this tab.
		 *
		 * Called by this tab's container.
		 *
		 * @param void
		 *
		 * @return void
		 * @overrides super
		 */
		uninitialize: function(){
		
			delete this.itemUUID;
			// Note: Do not delete the tab container (do not own)
			delete this.tabID;
			delete this.tabTitle;
			
			this.inherited(arguments);
		}
	
	});
	
})();
