/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn");

dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("jazz.ui.dnd.DraggableContainer");
dojo.require("jazz.ui.dnd.DraggableManager");
dojo.require("jazz.util.ResizeNotifier");

(function() {
	var draggableManager = jazz.ui.dnd.draggableManager();
	var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
	var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
	
	dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn", [jazz.ui.dnd.DraggableContainer, Observable], {
		constructor: function(node, params){
			this.avatarClass = "viewletColumnAvatar";
			this._columnHelper = params.columnHelper;
			this._viewletPage = params.viewletPage;
			this._disabled = true;   //default to disabled, until setEditMode(true) is called
			this.index = params.columnIndex;
			this.useAvatar = false;
			this.relativeTo = this._viewletPage.dashboardContainer.domNode;
			if (this.index===0) {
				this.avatarClass += " column1";
			}
			this.setEditMode(params.editMode, true);
		},
		dragStopEvent: function(event) {
			if (event.target) {
				var pn = event.target.parentNode;
				if (pn) {
					var ppn = event.target.parentNode.parentNode;
					if (dojo.hasClass(pn, 'header') || (ppn && dojo.hasClass(ppn, 'header'))) {
						dojo.stopEvent(event);
					}
				}
			}
			this.inherited(arguments);
		},
		onMouseDown: function(event) {
			if (this._disabled) {
				return;
			}
			
			this.inherited(arguments);
			
			var widget = draggableManager.widget;
			if (widget && widget.isZoomed) {
				var preCoords = dojo.coords(event.currentTarget, true);
				var coords = dojo.coords(widget.zoomPlaceholder, true);
				var offsetX = event.pageX - preCoords.x;
				var offsetY = event.pageY - preCoords.y; 
				if (offsetX > coords.w - 80) {
					offsetX = coords.w - 80;
				}
				draggableManager.initialMouseCoords.offsetX = offsetX;
				draggableManager.initialMouseCoords.offsetY = offsetY;
			}
		},
		onStartDrag: function() {
			var widget = draggableManager.widget;
			if (widget && widget.isZoomed) {
				draggableManager.widget.toggleZoom(null, true);
				var coords = dojo.coords(draggableManager.widget.domNode,true);
				draggableManager.widgetCoords.h = coords.h;
				draggableManager.widgetCoords.w = coords.w;
			}
			this._viewletPage.dashboardContainer._showBlockerDiv(0, 2000);
		},
		setEditMode: function(editing, noPropagate) {
			this._disabled = !editing;
			for(var i = 0; i< this.widgets.length; i++) {
				if (this.widgets[i] && this.widgets[i].setEditMode) {
					if(!noPropagate) this.widgets[i].setEditMode(editing);
					if (editing) {
						dojo.addClass(this.widgets[i].handle,"dndHandle");
					} else {
						dojo.removeClass(this.widgets[i].handle,"dndHandle");
					}
				}
			}
			if (editing) {
				dojo.removeClass(this.node,"dndContainerDisabled");
			} else {
				dojo.addClass(this.node,"dndContainerDisabled");
			}
		},
		add: function(widget, params) {
			this.inherited(arguments);
			if(this._disabled) {
				dojo.removeClass(widget.handle,"dndHandle");	
			}
			this.events.push(dojo.connect(widget.handle, "onclick", this, this.dragStopEvent));
		},
		_removeOpacity: function(widget) {
			widget.domNode.style.filter="";
		},
		/*
		 * End temporary code
		 */
		move: function(fromIndex, toContainer, toIndex) {
			try {
				var widget = draggableManager.widget;
				// update model
				var viewletHelper = widget.viewletHelper;
				var viewletPageHelper = this._viewletPage.viewletPageHelper;
				var srcColumnHelper = this._columnHelper;
				var destColumnHelper = viewletPageHelper.columns[toContainer.index];
				
				// remove from src column
				srcColumnHelper.viewlets.splice(fromIndex, 1);
				
				// add to destination column
				if (!destColumnHelper.viewlets) {
					destColumnHelper.viewlets = new Array();
				}
				destColumnHelper.viewlets.splice(toIndex, 0, viewletHelper);
				
				widget._columnIndex = toContainer.index;
				
				// notify listeners
				this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			} catch (err) {
				console.error("New viewlet position could not be saved: " + err.message + " - Refreshing viewlet page.");
				this._viewletPage.refreshPage();
			}
			this.inherited(arguments);
		},
		remove: function(index){
			var widget = this.widgets[index];
			// update model
			this._columnHelper.viewlets.splice(index, 1);
			// notify listeners that column has changed
			this.propertyChanged(Constants.PROPERTY_DASHBOARD);
			this.inherited(arguments);
			this._viewletPage.updateEmptyPageMessage();
		},
		onDndStop: function(success) {
			this.inherited(arguments);
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
			this._viewletPage.dashboardContainer._hideBlockerDiv();
		},
		_onChange: function(/* string */ property) {
			// notify listeners that the content has changed
			this.propertyChanged(property);
		}
	});
})();
