﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class ChartArea extends ChartObject {

	private var _debugArea:DebugArea;
	
	private var _legend:Legend;
	
	private var _chart:IChart;
	
	private var _layoutManager:LayoutManager;
	
	public static var FONT:String = "Arial";
	
	public static var MARGIN:Number = 5;	

	
	public function ChartArea() {
		super();
		
		// Initialize
		this._initialize();		
		
	}	
	
	private function _initialize():Void {
		
		var model:ChartModel = ChartEngine.getInstance().getModel();
		
		// Debug Area
		if (model.isDebug) {
			this._debugArea = new DebugArea();
		}
		
		// Chart
		if (model.presentation == "pie") {
			// Pie Chart
			this._chart = new PieChart();
			// Legend
			this._legend = new Legend();
		} else if (model.presentation == "column") {
			// Column Chart
			this._chart = new VerticalBarChart();
			// Legend
			this._legend = new Legend();
		} else {
			// Bar Chart
			this._chart = new HorizontalBarChart();
		}
		
		// Layout Manager
		this._layoutManager = new LayoutManager(this);
		
	}
	
	public function draw(parentMC:MovieClip) {

		this.mc = parentMC.createEmptyMovieClip("chart_area_mc", parentMC.getNextHighestDepth());	
		
		this._layoutManager.layout(this.mc);
	}
	
	public function getLegend():Legend {
		return this._legend;
	}
	
	public function getDebugArea():DebugArea {
		return this._debugArea;
	}	

	public function getChart():IChart {
		return this._chart;
	}	
	
	public function destroy():Void {
		// Debug area
		if (this._debugArea) {
			this._debugArea.destroy();
			delete this._debugArea;
		}
		// Legend
		if (this._legend) {
			this._legend.destroy();
			delete this._legend;
		}
		// Chart
		if (this._chart) {
			this._chart.destroy();
			delete this._chart;
		}
		// Layout manager
		if (this._layoutManager) {
			this._layoutManager.destroy();
			delete this._layoutManager;
		}
		
		super.destroy();
	}	
	
}