/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.query.JUnitRowElementViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.html");

dojo.require("com.ibm.team.build.web.ui.internal.Constants");
dojo.require("com.ibm.team.build.web.ui.internal.utils.TypeUtil");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.query.RowElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.query.JUnitCellElementViewer");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
(function() {

/*
 *  Summary: Inherit from the RowElementViewer row widget. This class render all the rows in the JUnit tree-table,
 *  		 including the parent-child relationship of the component-suite-suiteclass-testcase, and showing the detailed 
 * 			 errors and failures data.
 */

var Constants = com.ibm.team.build.web.ui.internal.Constants;
var TypeUtil = com.ibm.team.build.web.ui.internal.utils.TypeUtil;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var RowElementViewer = com.ibm.team.build.web.ui.internal.query.RowElementViewer;
var JUnitCellElementViewer = com.ibm.team.build.web.ui.internal.query.JUnitCellElementViewer; 
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.query.JUnitRowElementViewer",
 [RowElementViewer],
{     	
	initializeViewer: function(){
		if(this._rowType == Constants.JUNIT_COMPONENT_ROW ){ //Show the root rows in tree which type is "component"
			dojo.removeClass(this.domNode, "hidden"); //$NON-NLS-1$
		}
		if(this._rowType == Constants.JUNIT_CASE_ROW ){ //set the row which type is "test case" to be a leaf
			this._isLeafRow = true;
		}
	},
	
	createChildRows: function(){  
		var commonArgs = {  resultItemId: this._resultItemId,
						    callback: dojo.hitch(this, this._handleCallback),
						    errback: ErrorUtil.handleErrorResponseAlert };
		var args = null;
		var deferred;
		if(this._rowType == Constants.JUNIT_COMPONENT_ROW){
			args = dojo.mixin(commonArgs, { componentName: this._rowData.name } );
			deferred = BuildClient.getSuites(args);
		}
		if(this._rowType == Constants.JUNIT_SUITE_ROW){
			args = dojo.mixin(commonArgs, { suiteItemId: this._rowData.itemId} );
			deferred = BuildClient.getSuiteClasses(args);
		}
		if(this._rowType == Constants.JUNIT_CLASS_ROW){
			args = dojo.mixin(commonArgs, { classItemId: this._rowData.itemId} );
			deferred = BuildClient.getTestCases(args);
		}
		if (deferred) {
			deferred.addErrback(dojo.hitch(this,function(response) {
				if (this.domNode)
					this.loadFailed();
			}));
		}
	},
		
	_handleContentCallback: function(response){
		if (!this.domNode)
			return;
		this.populateDetails(response);
	},
	
	_handleCallback: function(response){
		if (!this.domNode)
			return;
		var rowType = TypeUtil.getChildrenRowType(this._rowType);
		this.populateColumns(response, rowType );
		return response;
	},
	
	populateColumns: function(rowData, rowType){
		var _columns = [];
		if(rowType == Constants.JUNIT_CASE_ROW){ //Test cases are the member of every test suite class
			rowData = rowData.testCases;
		}
		if( rowData && rowData.length > 0){
			var rowElements = new Array(rowData.length);
			for(var i = 0; i < rowData.length; i++){
				var self = this;
				if(rowType == Constants.JUNIT_CASE_ROW){ //Fake the row type for Test cases
					rowData[i][Constants.REPOSITORY_TYPE] = Constants.JUNIT_CASE_ROW;
				}				
				var rowArgs = { parentPage: self._parentPage,
								container: self._bodyContainer, 
								parentRowElement: self, 
								_resultItemId: self._resultItemId,
								_rowData: rowData[i] 
				};
				rowElements[i] = new com.ibm.team.build.web.ui.internal.query.JUnitRowElementViewer(rowArgs);
				
				var columnTypes = TypeUtil.getColumnTypes(rowType);
				
				for (var j = 0; j < columnTypes.length; j++) {
					var cellArgs = { rowContainer: rowElements[i],
									 cellType: columnTypes[j] };
					_columns[j] = new JUnitCellElementViewer(cellArgs);
					if (j == 0 && !rowElements[i].hasDetails())
						rowElements[i]._twistie = _columns[j].addTwistie();
					rowElements[i].domNode.appendChild(_columns[j].domNode);
				}
			}
			this.toggleExpand(rowElements);
		}
	},
	
	populateDetails: function(response){ //Show errors or warning info for rows which type is "Source File" 
		this._parentPage.getJUnitDetailsContainer().innerHTML = "<div class='junit-message'><pre>"+jazz.util.html.escape(response)+"</pre></div>";
	},
	
	prepareShowDetails: function(){
		this._parentPage.getJUnitDetailsContainer().innerHTML = ""; 
		if(this._rowData.error)
			this.setDetailType(Constants.JUNIT_ERROR_CONTENT_TYPE);
		if(this._rowData.failure)
			this.setDetailType(Constants.JUNIT_FAILURE_CONTENT_TYPE);
	},
	
	// TODO: We should show some loading indication while fetching details
	doShowDetails: function(){ 
		this.prepareShowDetails();
		if(this._rowType == Constants.JUNIT_CASE_ROW){
			var args =  { resultItemId: this._resultItemId,
				      classItemId: this.parentRowElement.getRowData().itemId,
					  caseName: this._rowData.name,
					  handleAs: "text",  //$NON-NLS-1$
					  callback: dojo.hitch(this, this._handleContentCallback),
					  errback: ErrorUtil.handleErrorResponseAlert  
					};
			if(this.getDetailType() == Constants.JUNIT_ERROR_CONTENT_TYPE)
				BuildClient.getErrorContent(args);
			if(this.getDetailType() == Constants.JUNIT_FAILURE_CONTENT_TYPE)
				BuildClient.getFailureContent(args);
		}			
	},
	
	destroy: function() {
		this.inherited(arguments);
	}
		
});
	
})();
