/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PersonaMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var HoverPopup = jazz.ui.HoverPopup;
var HTML = jazz.util.html;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var MemberInfoClient = com.ibm.team.dashboard.viewlets.web.client.internal.MemberInfoClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var SERVICE_PHOTO_ID = "com.ibm.team.dashboard.viewlets.service.internal.members.IMemberPhotoService";

var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.MemberInfo",
	[dijit._Widget, dijit._Templated], {
	
	// template contains nested divs to support including it in tooltips
	templateString: "<div><div class='com-ibm-team-dashboard-viewlets-web-ui-internal-utils-MemberInfo' dojoAttachPoint='_content'></div></div>",
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "PersonaMessages");
		this._dto = args.memberInfoDto;
		this._linkTeams = args.linkTeams;
		this._linkProjects = args.linkProjects;
	},
	
	postCreate: function() {
		this._populateContent();
	},
	
	_populateContent: function() {
		this._content.innerHTML = "";
		this._doPersonalArea();
		if (this._dto.processAreas && this._dto.processAreas.length > 0)
			this._doProcessAreas();
	},
	
	_doPersonalArea: function() {
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		
		var nameRow = document.createElement("tr");
		var imgSrc = Util.getServiceURL(SERVICE_PHOTO_ID, null, {userId: this._dto.userId});
		
		var photoCell = document.createElement("td");
		dojo.addClass(photoCell, "photo");
		photoCell.rowSpan = 3;
		var photo = document.createElement("img");
		photo.src = imgSrc;
		photo.width = "48";
		photo.height = "48";
		photoCell.appendChild(photo);
		nameRow.appendChild(photoCell);
		
		var nameLabelCell = document.createElement("td");
		dojo.addClass(nameLabelCell, "label");
		nameLabelCell.appendChild(document.createTextNode(messages.name));
		nameRow.appendChild(nameLabelCell);
		var nameCell = document.createElement("td");
		nameCell.appendChild(document.createTextNode(this._dto.name));
		nameRow.appendChild(nameCell);
		
		var idRow = document.createElement("tr");
		var idLabelCell = document.createElement("td");
		dojo.addClass(idLabelCell, "label");
		idLabelCell.innerHTML = messages.userId.replace(" ", "&nbsp;");
		idRow.appendChild(idLabelCell);
		var idCell = document.createElement("td");
		idCell.appendChild(document.createTextNode(this._dto.userId));
		idRow.appendChild(idCell);
		
		var emailRow = document.createElement("tr");
		var emailLabelCell = document.createElement("td");
		dojo.addClass(emailLabelCell, "label");
		emailLabelCell.appendChild(document.createTextNode(messages.email));
		emailRow.appendChild(emailLabelCell);
		var emailCell = document.createElement("td");
		var emailLink = document.createElement("a");
		emailLink.href = "mailto:" + this._dto.email;
		emailLink.innerHTML = this.insertWordBreaks(this._dto.email);
		emailCell.appendChild(emailLink);
		emailRow.appendChild(emailCell);
		
		tbody.appendChild(nameRow);
		tbody.appendChild(idRow);
		tbody.appendChild(emailRow);
		
		table.appendChild(tbody);
		
		this._content.appendChild(table);
	},
	
	insertWordBreaks: function(input) {
		if (!input)
			return "";
		var i, j, index, result, last;
		var e = jazz.util.html.escape;
		var delimeters = ["_","-",".","@"], segments = [input];
		for (i = 0; i < delimeters.length; i++) {
			result = [];
			last = i === delimeters.length - 1;
			for (j = 0; j < segments.length; j++) {
				while ((index = segments[j].indexOf(delimeters[i])) !== -1) {
					result.push(last ? e(segments[j].substring(0,index+1)) : segments[j].substring(0,index+1));
					segments[j] = segments[j].substring(index+1);
				}
				if (segments[j])
					result.push(last ? e(segments[j]) : segments[j]);
			}
			segments = result;
		}
		return segments.join("<wbr/>");
	},
	
	_doProcessAreas: function() {
		var ul = document.createElement("ul");
		dojo.addClass(ul, "processList");
		
		var title = document.createElement("li");
		dojo.addClass(title, "processTitle");
		title.appendChild(document.createTextNode(messages.process));
		ul.appendChild(title);
		
		for (var i in this._dto.processAreas) {
			this._doProcessArea(ul, this._dto.processAreas[i], 0, this._linkProjects);
		}
		
		this._content.appendChild(ul);
	},
	
	_doProcessArea: function(ul, area, indent, link) {
		var li = document.createElement("li");
		dojo.addClass(li, "processListEntry");
		var table = document.createElement("table");
		var tbody = document.createElement("tbody");
		var tr = document.createElement("tr");
		
		var td = document.createElement("td");
		td.style.paddingLeft = indent + "em";
		
		if (link) {
			var name = document.createElement("a");
			var isProject = !area.path;
			var scope = isProject ? DashboardConstants.SCOPE_PROJECT_AREA : DashboardConstants.SCOPE_TEAM_AREA;
			var scopeItem = isProject ? area.projectName : area.path
			name.href = DashboardUtil.getDashboardHref(scope, scopeItem, null, area.projectName);
		} else {
			var name = document.createElement("span");
		}
		name.appendChild(document.createTextNode(area.name));
		td.appendChild(name);
		tr.appendChild(td);
		
		if (area.roles && area.roles.length > 0) {
			td = document.createElement("td");
			dojo.addClass(td, "roles");
			for (var i in area.roles) {
				if (i != 0)
					td.appendChild(document.createTextNode(", "));
				var span = document.createElement("span");
				span.appendChild(document.createTextNode(area.roles[i]));
				new HoverPopup({
					around: span,
					persist: false,
					createContents: dojo.hitch(this,this._populateTooltip)}).data = {
						roleId: area.roles[i],
						processAreaScope: indent == 0 ? DashboardConstants.SCOPE_PROJECT_AREA : DashboardConstants.SCOPE_TEAM_AREA,
						processAreaItemId: area.itemId};
				td.appendChild(span);
			}
			tr.appendChild(td);
		}
		
		tbody.appendChild(tr);
		table.appendChild(tbody);
		li.appendChild(table);
		
		ul.appendChild(li);
		
		if (area.children) {
			for (var i in area.children) {
				this._doProcessArea(ul, area.children[i], indent + 1, this._linkTeams);
			}
		}
	},
	
	_populateTooltip: function(tip) {
		var d = document.createElement("div");
		d.className = "com-ibm-team-dashboard-viewlets-web-ui-internal-utils-MemberInfo-TooltipLoading";
		d.innerHTML = "Loading...";
		var serviceObj = {
			node: d,
			success: function(/* string */ description) {
				this.node.className = "com-ibm-team-dashboard-viewlets-web-ui-internal-utils-MemberInfo-Tooltip";
				if (description)
					this.node.innerHTML = HTML.escape(description);
				else
					this.node.innerHTML = messages.noDescription;
			},
			failure: function(error) {
				srh.doErrorCallback(error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		MemberInfoClient.getRoleDescription(tip.data.roleId, tip.data.processAreaScope, tip.data.processAreaItemId, srh);
		return d;
	}
});

})();