/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.actions.ItemMovePolicy"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.PostPlanSaveRunnable"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory"); //$NON-NLS-1$

(function() {
	
var Assert							= com.ibm.team.apt.web.ui.internal.common.Assert;
var PostPlanSaveRunnable			= com.ibm.team.apt.web.client.internal.PostPlanSaveRunnable;
var WorkItemSaveManager				= com.ibm.team.apt.web.client.internal.WorkItemSaveManager;

var WorkItemProxyFactory	= com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.actions.ItemMovePolicy", null, { //$NON-NLS-1$

	__viewModel: null,

	__viewMode: null,
	__groupProvider: null,
	__entrySorter: null,

	__getPlanModeFn: null,

	constructor: function(viewModel, getPlanModeFn) {
		this.__viewModel= viewModel;
		this.__getPlanModeFn= getPlanModeFn;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	setViewMode: function(viewMode) {
		this.__viewMode= viewMode;
	},

	setGroupProvider: function(groupProvider) {
		this.__groupProvider= groupProvider;
	},

	setEntrySorter: function(entrySorter) {
		this.__entrySorter= entrySorter;
	},

	canMove: function(sourceEntry, targetEntry, dropLocation) {
		return this.__viewModel.readModel(function(readAccessor) {
			var result= false;

			var request= this.__createItemMoveRequest(sourceEntry, targetEntry, dropLocation);
			if (request != null) {
				var handlers= this.__getHandlers();
				
				for (var i= 0; i < handlers.length; i++) {
					var response= handlers[i].canMove(request, readAccessor);
					if (response != null) {
						if (!response.canMove())
							return false;
						result= true;
					}
				}
			}

			return result;
		}, this);
	},
	
	move: function(sourceEntry, targetEntry, dropLocation) {
		this.__viewModel.updateModel(function(updateAccessor) {
			var n= updateAccessor.getEntryNavigator(true);

			var request= this.__createItemMoveRequest(sourceEntry, targetEntry, dropLocation);
			if (request != null) {
				var handlers= this.__getHandlers();
				var responses= [];
				var response;
				
				for (var i= 0; i < handlers.length; i++) {
					response= handlers[i].canMove(request, updateAccessor);
					responses.push(response);
					if (response != null && !response.canMove())
						return;
				}

				for (var i= 0; i < handlers.length; i++) {
					response= responses[i];
					if (response != null) {
						handlers[i].move(response, updateAccessor);
					}
				}
			}

			var planMode= this.__getPlanModeFn();
			var store= planMode.getStore();
			var oldStore= store.serialize();
			this.__viewMode.storeOptions(updateAccessor);
			this.__groupProvider.storeOptions(updateAccessor);
			this.__entrySorter.storeOptions(updateAccessor);
			var newStore= store.serialize();
			
			if (newStore != oldStore) {
				var plan= this.__viewModel.getInput();
				WorkItemSaveManager.save(new PostPlanSaveRunnable(plan, planMode));
			}
		}, this);
	},
	
	canCreate: function(targetEntry, workItemType, location) {
		var plan= this.__viewModel.getInput();
		if (!plan.getRootCategory())
			return false;
		
		if (plan.getIterations().length < 1)
			return false;
		
		return this.__viewModel.readModel(function(readAccessor) {
			var result= false;

			var request= this.__createItemCreateRequest(targetEntry, workItemType, location);
			if (request != null) {
				var handlers= this.__getHandlers();
				
				for (var i= 0; i < handlers.length; i++) {
					var response= handlers[i].canCreate(request, readAccessor);
					if (response != null) {
						if (!response.canCreate())
							return false;
						result= true;
					}
				}
			}

			return result;
		}, this);
	},
	
	create: function(targetEntry, workItemType, location) {
		var workingCopy= WorkItemProxyFactory.getWorkItemProxy({createIfNeeded: true});
		workingCopy.incrementUseCount();
		workingCopy.initialize({
			_self: this,
			type: workItemType.itemId,
			onSuccess: function() {
				this._self.__viewModel.updateModel(dojo.hitch(this._self, function(updateAccessor) {

					var request= this.__createItemCreateRequest(targetEntry, workItemType, location);
					if (request != null) {
						var handlers= this.__getHandlers();
						var responses= [];
						var response;
						
						for (var i= 0; i < handlers.length; i++) {
							response= handlers[i].canCreate(request, updateAccessor);
							responses.push(response);
							if (response != null && !response.canCreate()) {
								workingCopy.decrementUseCount(true);
								return;
							}
						}

						// see defect 81914, defect 81918
						WorkItemSaveManager.compoundSave(function() {
							var plan= this.__viewModel.getInput();
							var newItem;
							plan.compoundChange(function() {
								newItem= plan.createNewPlanItem(plan, workingCopy); 
								newItem.setAttributeValue(com.ibm.team.apt.client.PlanItem.TARGET, plan.getIterations()[0]);
								newItem.setAttributeValue(com.ibm.team.apt.client.PlanItem.CATEGORY, plan.getRootCategory());
							}, this);
	
							for (var i= 0; i < handlers.length; i++) {
								response= responses[i];
								if (response != null) {
									response.newItem= newItem;
									response.targetEntry= targetEntry;
									handlers[i].create(response, updateAccessor);
								}
							}
							
							var planMode= this.__getPlanModeFn();
							var store= planMode.getStore();
							var oldStore= store.serialize();
							this.__viewMode.storeOptions(updateAccessor);
							this.__groupProvider.storeOptions(updateAccessor);
							this.__entrySorter.storeOptions(updateAccessor);
							var newStore= store.serialize();
							
							if (newStore != oldStore) {
								WorkItemSaveManager.save(new PostPlanSaveRunnable(plan, planMode));
							}
							
							var n= updateAccessor.getEntryNavigator(false);
							var primaryEntry= n.findPrimaryEntry(newItem);
							if (primaryEntry != null) {
								updateAccessor.executeAfterUpdate(dojo.hitch(this, function() {
									this.__viewModel.revealEntry(primaryEntry);
								}));
							}
							
						}, this.__viewModel, {showErrorMessage: function() {}}, this);
					}
					
					return null;
				}));
			}, 
			onError: function(e) {
				workingCopy.decrementUseCount();
				console.log(e);
			}
		});
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__getHandlers: function() {
		var result= [];

		if (this.__viewMode != null) {
			result.push(this.__viewMode);
		}
		if (this.__groupProvider != null) {
			result.push(this.__groupProvider);
		}
		if (this.__entrySorter != null) {
			result.push(this.__entrySorter);
		}

		return result;
	},

	__createItemMoveRequest: function(sourceEntry, targetEntry, dropLocation) {
		return new com.ibm.team.apt.ui.Move.Request(sourceEntry.getElement(), sourceEntry, targetEntry, dropLocation);
	},
	
	__createItemCreateRequest: function(targetEntry, workItemType, location) {
		return new com.ibm.team.apt.ui.Create.Request(targetEntry, workItemType, location);
	},

	__sentinel: null // terminates this class definition
});

})();
