/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.EditableTableSection");
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TableEntry");
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.MoveableTableEntry");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");

(function() {

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var Term= com.ibm.team.workitem.web.client.internal.query.Term;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.EditableTableSection", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/EditableTableSection.html"),
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.title= args.title;
		this.textAdd = this.messages.buttonAdd;
		this.textRemove = this.messages.buttonRemove;
		this.textUp = this.messages.buttonUp;
		this.textDown = this.messages.buttonDown;
		this.addActionName= args.addActionName;
		this.emptyMessage= args.emptyMessage;
		this.supportOrdering= args.supportOrdering;
		
		this._entries= new Array();
		this._hasChanged= false;
	},
	
	postCreate: function() {
		this.textAdd = this.messages.buttonAdd;
		dojox.data.dom.replaceChildren(this._title, document.createTextNode(this.title));
		dojox.data.dom.replaceChildren(this._addLink, document.createTextNode(this.addActionName));
		dojox.data.dom.replaceChildren(this._emptyMessage, document.createTextNode(this.emptyMessage));
		
		this._updateActions();
	},
	destroy: function() {
			try {
				dojo.forEach(this._entries, function(entry) {
					if(entry.destroy) entry.destroy();
				});
			} catch (e) {
				
			}
		this.inherited(arguments);
	},
	getValues: function() {
		var entries= this._entries;
		var values= new Array();
		for (var i= 0; i < entries.length; i++) {
			values.push(entries[i].getItem())
		}
		return values;
	},
	
	addEntry: function(entry) {
		this._tableBody.appendChild(entry.domNode);
		this._entries.push(entry);
		this._setChanged(true);
		this._updateEmptyMessage();
	},
	
	deleteEntry: function(entry) {
		var idx= dojo.indexOf(this._entries, entry);
		if (idx > -1) {
			this._entries.splice(idx, 1);
			this._updateEmptyMessage();
		}
		if (entry === this._selectedEntry)
			this.selectEntry(null);
		entry.destroy();
		this._setChanged(true);
		this._updateActions();
	},
	
	selectEntry: function(entry) {
		var selectedEntry= this._selectedEntry;
		if (selectedEntry) {
			dojo.removeClass(selectedEntry.domNode, "selected");
		}
		this._selectedEntry= entry;
		if (entry)
			dojo.addClass(entry.domNode, "selected");
		this._updateActions();
	},
	
	deleteAllEntries: function() {
		var entries= this._entries;
		for (var i= 0; i < entries.length; i++) {
			entries[i].destroy();
		}
		entries.length= 0;
		this._selectedEntry= null;
		this._updateEmptyMessage();
		this._updateActions();
		this._setChanged(true);
	},
	
	moveDown: function(entry) {
		var entries= this._entries;
		var index= dojo.indexOf(entries, entry);
		if (index >= 0 && index < (entries.length - 1)) {
			// update model
			var next= entries[index+1];
			entries[index+1]= entry;
			entries[index]= next;
			
			// update view
			var domNode= entry.domNode;
			var parent= domNode.parentNode;
			var successor= (index < entries.length - 2) ? entries[index+2].domNode : null;

			var removed= parent.removeChild(domNode);
			parent.insertBefore(removed, successor);

			this._setChanged(true);
			this._updateActions();
		}
	},
	
	moveUp: function(entry) {
		var entries= this._entries;
		var index= dojo.indexOf(entries, entry);
		if (index > 0) {
			// update model
			var prev= entries[index-1];
			entries[index-1]= entry;
			entries[index]= prev;
			
			// update view
			var domNode= entry.domNode;
			var parent= domNode.parentNode;
			var predecessor= prev.domNode;

			var removed= parent.removeChild(domNode);
			parent.insertBefore(removed, predecessor);

			this._setChanged(true);
			this._updateActions();
		}
	},
	
	onAdd: function(event) {
		event.preventDefault();
	},
	
	onRemove: function(event) {
		event.preventDefault();
		if (this._selectedEntry) {
			this.deleteEntry(this._selectedEntry);
		}
	},
	
	onDown: function(event) {
		event.preventDefault();
		if (this._selectedEntry) {
			this.moveDown(this._selectedEntry);
		}
	},
	
	onUp: function(event) {
		event.preventDefault();
		if (this._selectedEntry) {
			this.moveUp(this._selectedEntry);
		}
	},
	
	hasChanged: function() {
		return this._hasChanged;
	},
	
	_setChanged: function(changed) {
		this._hasChanged= changed;
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_updateEmptyMessage: function() {
		this._emptyMessage.style.display= (this._entries.length == 0) ? "block" : "none";
	},
	
	_updateActions: function() {
		
		var selectedEntry= this._selectedEntry;
		var index= selectedEntry ? dojo.indexOf(this._entries, selectedEntry) : -1;
		
		this._setVisible(this._removeLink, index != -1);
		this._setVisible(this._upLink, this.supportOrdering && index > 0);
		this._setVisible(this._downLink,  this.supportOrdering && index >= 0 && index < this._entries.length - 1);
	},
	
	_setVisible: function(node, show) {
		node.style.display= show ? "inline" : "none";
	}
	
});

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.TableEntry", [dijit._Widget, dijit._Templated], {

	templateString: '<tr dojoAttachEvent="onclick: onSelect">' +
					'<td dojoAttachPoint="_label"></td>' +
					'<td class="action"><div dojoAttachPoint= "_deleteButton" class="deleteButton" dojoAttachEvent="onclick: onDelete"></div></td>' +
					'</tr>',
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
		this.item= args.item;
		this.labelProperty= args.labelProperty;
		this.section= args.section;
	},
	
	postCreate: function() {
		var label= this.item[this.labelProperty];
		this._deleteButton.title = this.messages.titleDelete;
		dojox.data.dom.replaceChildren(this._label, document.createTextNode(label));
	},
	
	getItem: function() {
		return this.item;
	},
	
	onSelect: function(event) {
		this.section.selectEntry(this);
	},
	
	onDelete: function(event) {
		this.section.deleteEntry(this);
	}
	
});

})();
