/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/

 // Use the jslint 'good things' default with the exception of: nomen: false (don't check variable names since dijit api uses _ which is against jslint rules) and forin
 /*jslint white: true, onevar: true, browser: true, undef: true, nomen: false, eqeqeq: true, plusplus: true, bitwise: true, regexp: true, strict: true, newcap: true, immed: true */
 /*global com, dijit, dojo, dojox, net, window, jazz */
 "use strict";
 
dojo.provide("com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.WorkItemLinksItem");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("dojo._base.html");
dojo.require("dojo.i18n");
dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");
dojo.require("dojox.string.BidiComplex");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dijit.form.Button");
// require work item page view to get the css that the internal work item picker depends on
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");

dojo.require("jazz.app.ResourceLink");
dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function () {
	var ScmUtil, ScmClient, PlatformUI, ServiceResponseHandler, SelectWorkItemDialog;
	ScmUtil	= com.ibm.team.scm.web.client.util;
	ScmClient = com.ibm.team.scm.web.client.internal.ScmClient;
	PlatformUI = net.jazz.ajax.ui.PlatformUI;
	ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
	SelectWorkItemDialog = com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog;
	
	/**
	 * Widget to get the name of a new folder.
	 */
	dojo.declare(
			"com.ibm.team.scm.web.ui.internal.widgets.resourcesEditor.WorkItemLinksItem", 
			[dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/widgets/resourcesEditor/templates/WorkItemLinksItem.html"),
		constructor: function (params) {
			this.workItems = []; // com.ibm.team.workitem.rest.dto:WorkItemSummaryDTO
			this.workItemActionNodes = []; // corresponding nodes
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
			this.workItemLinksName = this.messages.WorkItemLinksItem_labelLinks;
			this.workItemLinksRemoveLabel = this.messages.WorkItemLinksItem_labelRemove;
		},
		postCreate: function () {
		},
		startup: function () {
			var that = this;
			this.connect(this._addLink, "onclick", function (evt) {
				evt.preventDefault();
				that.showWorkItemPicker();
			});			
		},
		showWidget: function () {
		},
		showWorkItemPicker: function () {
			var action, params, that, dialog;
			that = this;
			action = {};
			params = {
				allowsMultiple: true,
				parentController: null, //this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				exclusionList: [],
				onOk: function (selectedWorkItems, action) {
					that.addWorkItems(selectedWorkItems);					
				},
				onClose: function () {
					//self.parentController.focus();
				},
				action: action
			};
			dialog = new SelectWorkItemDialog(params);
		},
		addWorkItems: function (newWorkItems) {
			var i;
			for (i = 0; i < newWorkItems.length; i += 1) {
				if (!this.isDuplicate(newWorkItems[i])) {
					this.workItems.push(newWorkItems[i]);
				}
			}
			this.updateWorkItems();
		},
		isDuplicate: function (newWorkItem) {
			var i;
			for (i = 0; i < this.workItems.length; i += 1) {
				if (newWorkItem.id === this.workItems[i].id) {
					return true;
				}
			}
			return false;
		},
		/**
		 * Return the selected work item UUIDs
		 */
		getWorkItemItemIds: function () {
			var i, result;
			result = [];
			for (i = 0; i < this.workItems.length; i += 1) {
				result.push(this.workItems[i].workItemItemId);
			}
			return result;
		},
		updateWorkItems: function () {
			var table, i, tbody, row, column1, column2, div, description, resourceUri, linkArgs, titleLink, iconImg, resourceLink;
			dojox.data.dom.removeChildren(this._workItemLinksList);
			table = document.createElement("table");
			table.className = "workitem-table";
			this._workItemLinksList.appendChild(table);
			tbody = document.createElement("tbody");
			table.appendChild(tbody);
			this.workItemActionNodes = [];
			for (i = 0; i < this.workItems.length; i += 1) {
				row = document.createElement("tr");
				tbody.appendChild(row);
				description = this.workItems[i].id + ": " + this.workItems[i].summary;
				
				column1 = document.createElement("td");
				row.appendChild(column1);
				// Note. For some reason the relative URI requires a redirect and
				// doesn't display in IE. bug 96215
				resourceUri = ScmUtil.getWorkItemURI(this.workItems[i].id);
				linkArgs = {lazyFetch: true, uri: resourceUri};
				titleLink = document.createElement("a");
				titleLink.href = resourceUri;
				iconImg = document.createElement("img");
				iconImg.src = this.workItems[i].typeIconURL;
				column1.appendChild(iconImg);
				titleLink.appendChild(document.createTextNode(description));
				column1.appendChild(titleLink);
				resourceLink = new jazz.app.ResourceLink(linkArgs, titleLink);

				column2 = document.createElement("td");
				div = document.createElement("div");
				div.className = "workitem-action";
				div.title = this.workItemLinksRemoveLabel;
				column2.appendChild(div);
				row.appendChild(column2);
				this.connect(div, "onclick", "onRemoveWorkItemClick");
				this.workItemActionNodes.push(div);
			}
		},
		onRemoveWorkItemClick: function (evt) {
			var target, targetIndex;
			evt.preventDefault();
			// normalize w3c (originalTarget) and srcElement (IE)
			target = evt.target || evt.srcElement;
			targetIndex = dojo.indexOf(this.workItemActionNodes, target);
			if (targetIndex === -1) {
				return;
			}
			this.workItems.splice(targetIndex, 1);
			this.updateWorkItems();
		}
	}
	);
}());