/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.page.RecordTypeManager");

(function(){

var cq = com.ibm.rational.cm.web;
var BASE_URL = "cqartifactdetails.cq";
var _defaultRecordType, _submitTypes, _allRecordTypes, _fieldsInfo;

dojo.declare(
	"com.ibm.rational.cm.web.ui.page.RecordTypeManager",
	null,
{
	constructor: function(){
		_defaultRecordType = null;
		_submitTypes = [];
		_allRecordTypes = [];
		_fieldsInfo = {};
	},
	
	/*
	 * Fetches default record type
	 * {
	 *	onBegin: function(),
	 *	onLoad: function(defaultRecordType, allRecordTypes){},
	 * 	onError: function(){},
	 * 	onHandle: function(){}
	 * }
	 */
	getDefaultRecordType: function(obj){
		if(!_defaultRecordType){
			cq.xhrGet({
				url: "cqartifactdetails.cq?action=GetDefaultRecordType",
				onBegin: obj.onBegin,
				load: function(response, ioArgs){
					_defaultRecordType = response.DefaultRecordType;
					obj.onLoad(_defaultRecordType);
				},
				handle: obj.onHandle,
				onCancel: obj.onCancel,
				error: obj.onError,
				handleError: true,
				preventCache: true // RATLC01054142 : New record list can not be refreshed instantly when switching to other userdb
			});
		}else{
			if(obj.onBegin){
				obj.onBegin();
			}
			obj.onLoad(_defaultRecordType);
			if(obj.onHandle){
				obj.onHandle();
			}
		}
	},
	
	getSubmitTypes: function(obj){
		if(!_submitTypes || _submitTypes.length === 0){
			cq.xhrGet({
				url: "cqartifactdetails.cq?action=GetSubmitTypes",
				onBegin: obj.onBegin,
				load: function(response, ioArgs){
					if(!response){
						console.error("GetSubmitTypes: data is empty");
						return;
					}
					if(response.STATUS && response.STATUS == "false"){
						alert(response.ERROR_MESSAGE);
						console.error("GetSubmitTypes: " + response.ERROR_MESSAGE);
						return;
					}

					_submitTypes = response.SubmitTypes;
					obj.onLoad(_submitTypes);
				},
				onCancel: obj.onCancel,
				error: obj.onError,
				handle: obj.onHandle,
				handleError: true,
				preventCache: true
			});
		}else{
			if(obj.onBegin){
				obj.onBegin();
			}
			obj.onLoad(_submitTypes);
			if(obj.onHandle){
				obj.onHandle();
			}
		}
		
	},
	
	/*
	 * Fetches all record types
	 * {
	 *	onBegin: function(),
	 *	onLoad: function(defaultRecordType, allRecordTypes){},
	 * 	onError: function(){},
	 * 	onHandle: function(){}
	 * }
	 */
	getRecordTypes: function(obj){
		if(!_allRecordTypes || _allRecordTypes.length === 0){
			cq.xhrGet({
				url: "cqquerywizard.cq",
				onBegin: obj.onBegin,
				load: function(response, ioArgs){
					_allRecordTypes = response.items;
					obj.onLoad(_allRecordTypes);
				},
				onCancel: obj.onCancel,
				error: obj.onError,
				handle: obj.onHandle,
				handleError: true,
				preventCache: true
			});
		}else{
			if(obj.onBegin){
				obj.onBegin();
			}
			obj.onLoad(_allRecordTypes);
			if(obj.onHandle){
				obj.onHandle();
			}
		}
	},
	
	/*
	 * Fetches display fields for record type
	 * {
	 *	onBegin: function(),
	 *	onLoad: function(fields){},
	 * 	onError: function(){},
	 * 	onHandle: function(){}
	 * }
	 */
	getFieldsForRecordType: function(obj){
		var recordType = obj.recordType;
		if(!recordType || recordType.length === 0){ return; }
		if(!_fieldsInfo[recordType]){
			cq.xhrGet({
				url: BASE_URL,
				content: {
					action: "GetFieldsForRecordType",
					recordType: recordType
				},
				onBegin: obj.onBegin,
				load: function(response, ioArgs){
					if(response && response.STATUS == "true"){
						_fieldsInfo[recordType] = response.fields;
						obj.onLoad(_fieldsInfo[recordType]);
					}else{
						alert(messages.ErrorGetFieldsForRecordType+recordType);
					}
				},
				onCancel: obj.onCancel,
				error: obj.onError,
				handle: obj.onHandle,
				handleError: true,
				preventCache: true
			});		
		}else{
			if(obj.onBegin){
				obj.onBegin();
			}
			obj.onLoad(_fieldsInfo[recordType]);
			if(obj.onHandle){
				obj.onHandle();
			}
		}
	},
	
	/*
	 * Returns an array of index corresponding to default display fields from a given array
	 * 	fieldsArray
	 * 		an array of objects that includes fields information
	 * 	getFieldName
	 * 		a function that returns field name from argument object
	 * 		if argument is obj = { name: "abc", value: "123" }
	 * 		getFieldName = function(obj){ return obj.name; }
	 */
	getDefaultFields: function(/* Array */ fieldsArray, /* Function */ getFieldName){
		var arr = [];
		var len = fieldsArray.length;
		if(len === 0){ return arr; };
		var field_1 = null;
		var field_2 = null;
		var fieldNameArray = (getFieldName && dojo.isFunction(getFieldName)) ?
				dojo.map(fieldsArray, getFieldName) : fieldsArray;
		dojo.forEach(fieldNameArray, function(item, idx){
			switch(item.toLowerCase()){
				case "id":
					field_1 = idx;
					break;
				case "name":
					if(!field_1){
						field_1 = idx;
					}
					break;
				case "headline":
					field_2 = idx;
					break;
				case "summary":
					if(!field_2){
						field_2 = idx;
					}
					break;
			}
		});

		// if fields names didn't match by above logic, use the first 2 fields
		if(field_1 == null){
			field_1 = 0;
			if(field_2 === 0 && len > 1){
				field_2 = 1;
			}
		}
		if(field_2 == null){
			if(len > 1){
				if(field_1 == 0){
					field_2 = 1;
				}else{
					field_2 = 0;
				}
			}
		}
		if(field_1 != null){
			arr.push(field_1);
		}
		if(field_2 != null){
			arr.push(field_2);
		}
		return arr;
	}
});

})();

com.ibm.rational.cm.web.ui.page.recTypeManager = new com.ibm.rational.cm.web.ui.page.RecordTypeManager();
