/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009. All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.common.FloatingPane");

dojo.require("dojox.layout.FloatingPane");
dojo.require("dijit.form._FormWidget");

(function(){

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.FloatingPane",
	dojox.layout.FloatingPane,
{
	activeSizing: false,
	destroyOnClose: true, // RATLC01239978
	containerId: "", // ID of a node to constraint drag move
	
	// RATLC01062757 : The floating pane opened by "Open in New Window" can be dragged outside the browse area
	// override to constrain to container
	postCreate: function(){
		dojo.addClass(this.domNode, "com-ibm-rcm-FloatingPane");
		
		this.setTitle(this.title);

		dojox.layout.ContentPane.superclass.postCreate.call(this);

		var containerId = (this.containerId && this.containerId.length > 0) ? this.containerId : "cqShellContainer";
		this._mover = new dojo.dnd.move.parentConstrainedMoveable(this.domNode, {
			handle: this.focusNode,
			area: containerId,
			within: true
		});

		if(!this.dockable){ this.dockNode.style.display = "none"; } 
		if(!this.closable){ this.closeNode.style.display = "none"; } 
		if(!this.maxable){
			this.maxNode.style.display = "none";
			this.restoreNode.style.display = "none";
		}
		if(!this.resizable){
			this.resizeHandle.style.display = "none"; 	
		}else{
			var foo = dojo.marginBox(this.domNode); 
			this.domNode.style.width = foo.w+"px"; 
		}		
		this._allFPs.push(this);

		// RATLC01268828 : Help for Customized Columns is not readable
		this.bgIframe = new dijit.BackgroundIframe(this.domNode);
	},
	
	destroy: function(/*Boolean*/ finalize){
		this._mover.destroy(); // RATLC01063753 : Memory leaks in Record details view when using open new window
		if(this._dockNode){ this._dockNode.destroy(); }
		if(this._resizer){ this._resizer.destroy(); }
		if(this.bgIframe){ this.bgIframe.destroy(); }
		this.inherited("destroy", arguments);
	},

	// override so that it used CqDock
	startup: function(){
		dojox.layout.ContentPane.superclass.startup.call(this);

		if(this.resizable){
			if(dojo.isIE){
				this.canvas.style.overflow = "auto";
			}else{
				this.containerNode.style.overflow = "auto";
			}
			this._resizer = new dojox.layout.ResizeHandle({ // just use dojox.layout.ResizeHandle 1.2.0
				targetId: this.id
			},this.resizeHandle);
		}

		if(this.dockable){ 
			// FIXME: argh.
			tmpName = this.dockTo; 

			if(this.dockTo){
				this.dockTo = dijit.byId(this.dockTo); 
			}else{
				this.dockTo = dijit.byId('dojoxGlobalFloatingDock');
			}

			if(!this.dockTo){
				if(tmpName){ 
					var tmpId = tmpName;
					var tmpNode = dojo.byId(tmpName); 
				}else{
					var tmpNode = dojo.doc.createElement('div');
					dojo.body().appendChild(tmpNode);
					dojo.addClass(tmpNode,"dojoxFloatingDockDefault");
					var tmpId = 'dojoxGlobalFloatingDock';
				}
				this.dockTo = new com.ibm.rational.cm.web.ui.view.common.CqDock({ id: tmpId, autoPosition: "south" },tmpNode);
				this.dockTo.startup(); 
			}
			
			if((this.domNode.style.display == "none")||(this.domNode.style.visibility == "hidden")){
				// If the FP is created dockable and non-visible, start up docked.
				this.minimize();
			} 
		} 		
		this.connect(this.focusNode, "onmousedown", "bringToTop");
		this.connect(this.domNode, "onmousedown", "bringToTop");
	},

	// RATLC01052314 : CQ7.1 508 Falcon - Site Administration Configuration Dialog does not recieve focus
	show: function(/* Function? */callback){
		// summary: basically copied from show() of dojox.layout.FloatingPane
		// commented the line this.dockTo._positionDock(null); to fix a defect
		// in case this.dockTo is null
		var node = this.domNode;
		// RATLC01063753 : Memory leaks in Record details view when using open new window
		// don't use fadeIn since it causes memory leaks
//		var anim = dojo.fadeIn({
//			node: node,
//			duration: this.duration,
//			beforeBegin: dojo.hitch(this, function(){
				node.style.display = ""; 
				node.style.visibility = "visible";
				
				////////////// comment following line. 
				//this.dockTo._positionDock(null);
				////////////////////////////////////
				
				if(this.dockTo){ this.dockTo._positionDock(null); }
				if(typeof callback == "function"){ callback(); }
				this._isDocked = false;
				if(this._dockNode){ 
					this._dockNode.destroy();
					this._dockNode = null;
				}
//			})
//		}).play();
		this.resize(dojo.coords(node));
		
		this._keypressconnect = this.connect(dojo.doc, "onkeypress", "_onKey");

		this._savedFocus = dijit.getFocus(this);

		// set timeout to allow the browser to render dialog
		setTimeout(dojo.hitch(this, function(){
			dijit.focus(this.focusNode);
		}), 50);

		// RATLC01268828 : Help for Customized Columns is not readable
		var bgi = this.bgIframe;
		if(bgi && bgi.iframe){
			bgi.iframe.style.display = "block";
		}
	},
	
	hide: function(/* Function? */ callback){
		// RATLC01063753 : Memory leaks in Record details view when using open new window
		// don't use fadeOut since it causes memory leaks
		// summary: close but do not destroy this widget
//		dojo.fadeOut({
//			node:this.domNode,
//			duration:this.duration,
//			onEnd: dojo.hitch(this,function() { 
				this.domNode.style.display = "none";
				this.domNode.style.visibility = "hidden"; 
				if(this.dockTo){
					this.dockTo._positionDock(null);
				}
				if(callback){
					callback();
				}
//			})
//		}).play();

		// RATLC01268828 : Help for Customized Columns is not readable
		var bgi = this.bgIframe;
		if(bgi && bgi.iframe){
			bgi.iframe.style.display = "none";
		}
	},

	// RATLC01239978, override dojox.layout.FloatingPane
	close: function(){
		if(!this.closable){ return; }
		dojo.unsubscribe(this._listener);
		if(this.destroyOnClose){
			this.hide(dojo.hitch(this,"destroy"));
		}else{
			this.hide();
		}
	},
	
	// Keyboard accessibility
	// ESC : close
	// Ctrl + Alt + n : minimize
	// Ctrl + Alt + x : maximize
	// Ctrl + Alt + r : restore
	_onKey: function(/*Event*/ evt){
		if(evt.keyCode == dojo.keys.ESCAPE){
			var node = evt.target;
			while(node){
				if(node == this.domNode){
					this.close(); 
				}
				node = node.parentNode;
			}
		}else{
			if(evt.ctrlKey && evt.altKey){
				switch(evt.keyChar){
					case "n":
						this.minimize();
						break;
					case "x":
						this.maximize();
						break;
					case "r":
						this._restore();
						break;
				}
			}
		}
	},

	_dock: function(){
		this.inherited("_dock", arguments);
		
		// set focus to dockNode
		var node = null;
		if(this._isDocked){
			node = this._dockNode.domNode;
		}else{
			var savedFocus = this._savedFocus;
			if(savedFocus && savedFocus.node){
				node = savedFocus.node;
			}
		}
		setTimeout(dojo.hitch(this, function(){
			this.dockTo.setFocusDockNode(node);
		}), 500);
	},

	// This class overrides dojox.layout.FloatingPane just for the workaround of dojo's bug (dim.t and dim.l is not passed in IE, so cannot resize)
	// The defect has been submitted (https://cs.opensource.ibm.com/tracker/index.php?func=detail&aid=76658&group_id=2078&atid=5124)
	// and the fix is already delivered in trunk
	resize: function(/* Object */dim){
		this._currentState = dim;
		var dns = this.domNode.style;
		var str_px = "px";
		if(dim.t != null){ dns.top = dim.t+str_px; }
		if(dim.l != null){ dns.left = dim.l+str_px; }

		dns.width = dim.w+str_px; 
		this.canvas.style.width = dim.w+str_px;

		dns.height = dim.h+str_px;
		this.canvas.style.height = (dim.h - this.focusNode.offsetHeight)+str_px;
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.CqDock",
	dojox.layout.Dock,
{
	_focusDockNode: null, // DomNode
	
	// override so that it uses this.connect
	// and connects to onkeypress
	startup: function(){
		// accessibility
		this.connect(this.domNode, "onkeypress", "_onKeyPress");
	
		// summary: attaches some event listeners 
		if(this.id == "dojoxGlobalFloatingDock" || this.isFixedDock){
			// RATLC01236120 : Script error occurs when there is a floating pane and browser window is resized
			// attach window.onScroll, and a position like in presentation/dialog
			this.connect(window,'onresize',"_positionDock");
			this.connect(window,'onscroll',"_positionDock");
			if(dojo.isIE){
				this.connect(this.domNode, 'onresize', "_positionDock");
			}
		}
		this._positionDock(null);
	},

	setFocusDockNode: function(/* DomNode */ node){
		dijit.focus(node);
		this._focusDockNode = node; // custom property
	},
	
	_onKeyPress: function(e){
		var node = this._focusDockNode;
		if(!node){ return; }
		var childNodes = this.containerNode.childNodes;
		var len = childNodes.length;
		var index = dojo.indexOf(childNodes, node);
		var keys = dojo.keys;
		switch(e.keyCode){
			case keys.LEFT_ARROW:
				if(index > 0){
					index--;
				}else{
					index = len - 1;
				}
				this.setFocusDockNode(childNodes[index]);
				break;
			case keys.RIGHT_ARROW:
				if(index < len - 1){
					index++;
				}else{
					index = 0;
				}
				this.setFocusDockNode(childNodes[index]);
				dojo.stopEvent(e); // without this, focus will be lost
				break;
		}
	},
	
	// override in order to use com.ibm.rational.cm.web.ui.view.common.CqDockNode
	addNode: function(refNode){
		var dockNode = new com.ibm.rational.cm.web.ui.view.common.CqDockNode({ title: refNode.title, paneRef: refNode });
		this.containerNode.appendChild(dockNode.domNode);
		dockNode.startup();
		return dockNode;
	},
	
	// override in order to fix typo
	_positionDock: function(/* Event? */e){
		if(!this._inPositioning){	
			if(this.autoPosition == "south"){
				// Give some time for scrollbars to appear/disappear
				var str_px = "px";
				setTimeout(dojo.hitch(this, function(){
					this._inPositioning = true; // fix typo
					var viewport = dijit.getViewport();
					var s = this.domNode.style;
					s.left = viewport.l + str_px;
					s.width = (viewport.w-2) + str_px;
					s.top = (viewport.h + viewport.t) - this.domNode.offsetHeight + str_px;
					this._inPositioning = false;
				}), 500);
			}
		}
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.CqDockNode",
	[ dijit.form._FormWidget, dojox.layout._DockNode ],
{
	baseClass: "cqDockNode",
	
	postCreate: function(){
		this.inherited("postCreate", arguments);
		
		// accessibility
		var node = this.domNode;
		node.setAttribute("tabIndex", "0"); // "i" must be capitalized otherwise does not work in IE
		this.connect(node, "onmouseenter", "_onMouse");
		this.connect(node, "onmouseleave", "_onMouse");
		var self = this;
		this.connect(node, "onkeypress", function(e){
			if(e.keyCode == dojo.keys.ENTER){
				self.restore();
			}
		});
	}
});

})();