/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.navigation.NavigatorView");

dojo.require("jazz.util.format");
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.view.common.Tree");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "FavoritesMessages");

(function(){

var FormatUtil = jazz.util.format;
var cq = com.ibm.rational.cm.web;
var cqLoginController = com.ibm.rational.cm.cqLoginController;
var messages;

/*
 * Subclass of dijit.TreeNode in order to support
 * - Add anchor tag to execute queries when node is clicked
 * - Add "Add to ClearQuest favorites" icon at the right of tree node
 * - Make hover style consistent with WorkItem (round rectangle) 
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.navigation.NavigatorTreeNode",
	com.ibm.rational.cm.web.ui.view.common.TreeNode,
{
	// - add anchor tag (rcmAnchor)
	// - remove dojoAttachEvent of _onMouseEnter/_onMouseLeave from to rowNode
	templateString:'<div class="dijitTreeNode" waiRole="presentation"><a dojoAttachPoint="rcmAnchor" tabindex="-1" class="section-container"><div dojoAttachPoint="rowNode" class="dijitTreeRow" waiRole="presentation"><img src="${_blankGif}" alt="" dojoAttachPoint="expandoNode" class="dijitTreeExpando" waiRole="presentation"><span dojoAttachPoint="expandoNodeText" class="dijitExpandoText" waiRole="presentation"></span><span dojoAttachPoint="contentNode" class="dijitTreeContent" waiRole="presentation"><img src="${_blankGif}" alt="" dojoAttachPoint="iconNode" class="dijitTreeIcon" waiRole="presentation"><span dojoAttachPoint="labelNode" class="dijitTreeLabel" wairole="treeitem" tabindex="-1" waiState="selected-false" dojoAttachEvent="onfocus:_onNodeFocus"></span></span></div></a><div dojoAttachPoint="containerNode" class="dijitTreeContainer" waiRole="presentation" style="display: none;"></div></div>',		
	
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "FavoritesMessages");
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		// set action URL to anchor element
		var store = this.tree.model.store;
		var item = this.item;
		
		// hidden root is not an item and it causes script error,
		// and URL shouldn't need to be set for folders (causes error for empty folders)
		if(store.isItem(item) && !this.isExpandable){
			this.rcmAnchor.href = cq.getActionHref(store.getValue(item, "type"), store.getIdentity(item), this.tree.getLabel(item));
		}
		
		// add action icons for nodes other than folder
		if(!this.isExpandable){
			var actionAnchor = dojo.doc.createElement("A");
			actionAnchor.title = messages.AddToFavorites;
			dojo.addClass(actionAnchor, "artifactCommand");
			dojo.addClass(actionAnchor, "artifactHoverCommand");
			// 79085, add a DIV within A, according to the change in Foundation Web UI
			var actionDIV = dojo.doc.createElement("DIV");
			dojo.addClass(actionDIV, "add-favorite-command");
			actionAnchor.appendChild(actionDIV);
			var self = this;
			this.connect(actionAnchor, "onclick", function(e){
				dojo["require"]("com.ibm.rational.cm.web.ui.page.FavoritesController");
				com.ibm.rational.cm.favoritesController.addFavorite({
					type: store.getValue(item, "type"),
					title: self.tree.getLabel(item),
					stableLocation: store.getIdentity(item),
					masterReplica: "<local>" // TODO : hard-coded
				});
				dojo.stopEvent(e);
			});
			this.rowNode.appendChild(actionAnchor);
		}
	},
	
	// override so that label wraps for long strings
	setLabelNode: function(label){
		this.labelNode.innerHTML = (label) ? FormatUtil.insertWordBreaks(label, 18) : "";
	}
});

/*
 * Navigator view widget
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.navigation.NavigatorView",
	com.ibm.rational.cm.web.ui.view.common.Tree,
{
	postCreate: function(){
		this.filter = {
			type: [ "LIST", "CHART", "REPORT" ]
		}
		this.inherited(arguments);
		dojo.addClass(this.domNode, "com-ibm-rcm-NavigatorView");
	},
	
	/*
	 * Populate navigator tree
	 */
	initialize: function(){
		// check if it is already initialized
		var loadFinished = false;
		if(this.model){
			if(this.model.store){
				loadFinished = this.model.store._loadFinished;
			}
		}

		if(loadFinished){ return; }
		
		var self = this;
		cqLoginController.process({
			callback: function(){
				self._load();
			}
		});
	},
	
	// don't stopEvent when clicking nodes other than folder
	_onClick: function(/*Event*/ e){
		// summary: translates click events into commands for the controller to process
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}

		if( (this.openOnClick && nodeWidget.isExpandable) ||
			(domElement == nodeWidget.expandoNode || domElement == nodeWidget.expandoNodeText) ){
			// expando node was clicked, or label of a folder node was clicked; open it
			if(nodeWidget.isExpandable){
				this._onExpandoClick({node:nodeWidget});
			}
			dojo.stopEvent(e); // RCM
		}else{
			this._publish("execute", { item: nodeWidget.item, node: nodeWidget} );
			this.onClick(nodeWidget.item, nodeWidget);
			this.focusNode(nodeWidget);
		}
//		dojo.stopEvent(e); // RCM
	},
	
	// don't stopEvent when pressing ENTER key
	_onKeyPress: function(/*Event*/ e){
		// summary: translates keypress events into commands for the controller
		if(e.altKey){ return; }
		var dk = dojo.keys;
		var treeNode = dijit.getEnclosingWidget(e.target);
		if(!treeNode){ return; }

		var key = e.charOrCode;
		if(typeof key == "string"){  // handle printables (letter navigation)
			// Check for key navigation.
			if(!e.altKey && !e.ctrlKey && !e.shiftKey && !e.metaKey){
				this._onLetterKeyNav( { node: treeNode, key: key.toLowerCase() } );
				dojo.stopEvent(e);
			}
		}else{  // handle non-printables (arrow keys)
			var map = this._keyHandlerMap;
			if(!map){
				// setup table mapping keys to events
				map = {};
				map[dk.ENTER]="_onEnterKey";
				map[this.isLeftToRight() ? dk.LEFT_ARROW : dk.RIGHT_ARROW]="_onLeftArrow";
				map[this.isLeftToRight() ? dk.RIGHT_ARROW : dk.LEFT_ARROW]="_onRightArrow";
				map[dk.UP_ARROW]="_onUpArrow";
				map[dk.DOWN_ARROW]="_onDownArrow";
				map[dk.HOME]="_onHomeKey";
				map[dk.END]="_onEndKey";
				this._keyHandlerMap = map;
			}
			if(this._keyHandlerMap[key]){
				this[this._keyHandlerMap[key]]( { node: treeNode, item: treeNode.item } );
				if(key != dk.ENTER){ // RCM
					dojo.stopEvent(e);
				}
			}
		}
	},
		
	// override to use com.ibm.rational.cm.web.ui.view.navigation.NavigatorNode
	_createTreeNode: function(/*Object*/ args){
		return new com.ibm.rational.cm.web.ui.view.navigation.NavigatorTreeNode(args);
	}
});

})();