/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2008  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqDateTextBox");
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqTimeTextBox");
dojo.provide("com.ibm.rational.cm.web.ui.view.record.CqDateTimeTextBox");

dojo.require("com.ibm.rational.cm.web.ui.view.record.CqRecord");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");
dojo.require("dijit.layout.SplitContainer");

/*
 * Widgets for DATE_TIME fields used in CqRecord
 */
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqDateTextBox",
	[dijit.form.DateTextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin],
	{
		postMixInProperties:function(){
			this.inherited(arguments);
			var formDate = this.formDate || {};
			var dateCode = formDate.dateCode || "1"; // default is short
			dojo.mixin(this.constraints, {formatLength: dateCode == "1" ? "short" : "long"}); 
		},
		setValue: function(/*String*/ value, /*Boolean, optional*/ priorityChange){
			if(dojo.isString(value)){
				arguments[0] = value ? dojo.date.stamp.fromISOString(value) : null;
			}
			this.inherited(arguments);
		},
		setFieldData: function(data){
			this.inherited(arguments);
			// keep time part
			var v = this._lastValue;
			if(v){
				this._orgh = v.getHours(); this._orgm = v.getMinutes(); this._orgs = v.getSeconds();
			}
		},
		getFieldValue: function(){
			var val = this.getValue();
			if(!val){
				return "";
			}
			if(this._orgh){
				// restore time part
				val.setHours(this._orgh); val.setMinutes(this._orgm); val.setSeconds(this._orgs);
			}
			return dojo.date.stamp.toISOString(val, {zulu: true});
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqTimeTextBox",
	[dijit.form.TimeTextBox, com.ibm.rational.cm.web.ui.view.record.CqFormControl, com.ibm.rational.cm.web.ui.view.record.TextBoxResizeMixin],
	{
		postMixInProperties:function(){
			this.inherited(arguments);
			var formDate = this.formDate || {};
			var timeCode = formDate.timeCode || "1"; // default is short
			dojo.mixin(this.constraints, {formatLength: timeCode == "1" ? "medium" : "long"}); 
		},
		setValue: function(/*String*/ value, /*Boolean, optional*/ priorityChange){
			if(dojo.isString(value)){
				arguments[0] = value ? dojo.date.stamp.fromISOString(value) : null;
			}
			this.inherited(arguments);
		},
		setFieldData: function(data){
			this.inherited(arguments);
			// keep date part
			var v = this._lastValue;
			if(v){
				this._orgy = v.getFullYear(); this._orgm = v.getMonth(); this._orgd = v.getDate();
			}
		},
		getFieldValue: function(){
			var val = this.getValue();
			if(!val){
				return "";
			}
			if(this._orgy){
				// restore date part
				val.setFullYear(this._orgy); val.setMonth(this._orgm); val.setDate(this._orgd);
			}
			return dojo.date.stamp.toISOString(val, {zulu: true});
		}
	}
);

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.record.CqDateTimeTextBox",
	[dijit._Widget, dijit._Templated, com.ibm.rational.cm.web.ui.view.record.CqFormControl],
	{
		widgetsInTemplate: true,
		templateString: "<div><div dojoAttachPoint='container' dojoType='dijit.layout.SplitContainer' sizerWidth='1' persist='false'><div dojoType='dijit.form.DateTextBox' dojoAttachPoint='dateField'></div>" +
						"<div dojoType='dijit.form.TimeTextBox' dojoAttachPoint='timeField'></div></div></div>",
		postCreate:function(){
			var formDate = this.formDate || {};
			var dateCode = formDate.dateCode || "1"; // default is short
			var timeCode = formDate.timeCode || "1"; // default is short
			if(dateCode == "0" && timeCode == "0") {
				dateCode = timeCode = "1"; 
			}
			dojo.mixin(this.dateField.constraints, {formatLength: dateCode == "1" ? "short" : "long"}); 
			dojo.mixin(this.timeField.constraints, {formatLength: timeCode == "1" ? "medium" : "long"}); 
			this.connect(this.dateField, "onChange", "onDateChange");
			this.connect(this.timeField, "onChange", "onTimeChange");
			this.inherited(arguments);
		},
		onDateChange: function(v){
			if(v && !this.timeField.getValue()){
				this.timeField.setValue(new Date());
			}else{
				this.onChange(this.getValue());
			}
		},
		onTimeChange: function(v){
			if(v && !this.dateField.getValue()){
				this.dateField.setValue(new Date());
			}else{
				this.onChange(this.getValue());
			}
		},
		setDisabled: function(disabled){
		},
		setSize: function(w, h){
			this.inherited(arguments);
			this.container.domNode.style.width=(w-2)+"px";
			this.container.domNode.style.height=(h-2)+"px";
		},
		startup: function(){
			this.container.resize();
		},
		setValue: function(/*String*/ value, /*Boolean, optional*/ priorityChange){
			if(value){
				value = dojo.date.stamp.fromISOString(value);
			}
			this.dateField.setValue(value, priorityChange);
			this.timeField.setValue(value, priorityChange);
		},
		getValue: function() {
			var date = this.dateField.getValue();
			var time = this.timeField.getValue();
			if(!date && !time){
				return null;
			}
			if(!date) date = new Date(); // today
			if(!time) time = new Date(); // now
			return dojo.date.stamp.toISOString(new Date(date.getFullYear(), date.getMonth(), date.getDate(), time.getHours(), time.getMinutes(), time.getSeconds()), {zulu: true}); 
		}
	}
);
