/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.PlanDuration"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.Duration"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;
var Duration						= com.ibm.team.apt.client.Duration;

var UNSPECIFIED= -1;
var INVALID= -2;
var INVALID_TIME_SPENT_OFFSET= 3;

dojo.declare("com.ibm.team.apt.client.PlanDuration", com.ibm.team.apt.client.Duration, { //$NON-NLS-1$
	
	__timeSpent: null,
	__correctedEstimate: null,
	__originalEstimate: null,
		
	constructor: function(value/*super class*/, originalEstimate, correctedEstimate, timeSpent, isResolved){
		this.__originalEstimate= originalEstimate instanceof Duration ? originalEstimate : new Duration(originalEstimate);
		this.__correctedEstimate= correctedEstimate instanceof Duration ? correctedEstimate : new Duration(correctedEstimate);
		this.__timeSpent= timeSpent instanceof Duration ? timeSpent : new Duration(timeSpent);
	},
	
	// ---- api ------------------------------------------------------------------------------------------------------------

	getOriginalEstimate: function() {
		return this.__originalEstimate;
	},

	getCorrectedEstimate: function() {
		return this.__correctedEstimate;
	},

	getCurrentEstimate: function() {
		return this.__correctedEstimate.specified ? this.__correctedEstimate : this.__originalEstimate;
	},

	getTimeSpent: function() {
		return this.__timeSpent;
	},
	
	getTimeRemaining: function() {
		if(com.ibm.team.apt.client.PlanDuration.isEncodedInvalidTimeRemaining(this.__timeSpent)) {
			return com.ibm.team.apt.client.PlanDuration.decodeInvalidTimeRemaining(this.__timeSpent);
		} else if (this.__timeSpent.specified) {
			var resultMillis= this.getCurrentEstimate().__duration - this.__timeSpent.__duration;
			if(resultMillis >= 0) {
				return new Duration(resultMillis);
			} else {
				return Duration.INVALID;
			}
		} else {
			return this.__timeSpent; /* UNSPECIFIED or INVALID - all other cases are handles above */
		}
	},

	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;
		
		if (!this.inherited(arguments))
			return false;

		if (!this.__originalEstimate.equals(other.__originalEstimate))
			return false;
		if (!this.__correctedEstimate.equals(other.__correctedEstimate))
			return false;
		if (!this.__timeSpent.equals(other.__timeSpent))
			return false;

		return true;
	},

	
	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.client.PlanDuration.isTimeRemainingMode= function() {
}

com.ibm.team.apt.client.PlanDuration.create= function(originalEstimate, correctedEstimate, timeSpent, isResolved) {
	if (com.ibm.team.apt.client.PlanDuration.isEncodedInvalidTimeRemaining(timeSpent)) {
		timeSpent= com.ibm.team.apt.client.PlanDuration.decodeInvalidTimeRemaining(timeSpent);
	}
	
	var value= originalEstimate.__duration;
	if (correctedEstimate.specified)
		value= correctedEstimate.__duration;
	if (timeSpent.specified) {
		if (!isResolved) {
			value= Math.max(value - timeSpent.__duration, 0);
		} else {
			value= timeSpent.__duration;
		}
	}
	return new com.ibm.team.apt.client.PlanDuration(value, originalEstimate, correctedEstimate, timeSpent);
};

com.ibm.team.apt.client.PlanDuration.encodeInvalidTimeRemaining= function(duration) {
	Assert.isLegal(duration.__duration >= 0);
	var result= 0;
	result+= INVALID_TIME_SPENT_OFFSET;
	result+= duration.__duration;
	result*= -1;
	return new Duration(result);
};

com.ibm.team.apt.client.PlanDuration.decodeInvalidTimeRemaining= function(duration) {
	Assert.isLegal(com.ibm.team.apt.client.PlanDuration.isEncodedInvalidTimeRemaining(duration));
	result= duration.__duration;
	result*= -1;
	result-= INVALID_TIME_SPENT_OFFSET;
	return new Duration(result);
};

com.ibm.team.apt.client.PlanDuration.isEncodedInvalidTimeRemaining= function(duration) {
	return duration.__duration <= -INVALID_TIME_SPENT_OFFSET;
};

})();
