/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.plancheck.PlanCheckEngine"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("com.ibm.team.foundation.common.ProgressMonitor"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.common.Severity"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {


var PlatformUI						= net.jazz.ajax.ui.PlatformUI;
var PlanModelCheckReport			= com.ibm.team.apt.client.internal.plancheck.PlanModelCheckReport;
var ProgressMonitor					= com.ibm.team.foundation.common.ProgressMonitor;

var Severity						= com.ibm.team.apt.common.Severity;

var bind= dojo.string.substitute;
var Messages= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

// we cannot import com.ibm.team.apt.client.plancheck.PlanCheck now, as it is contributed by the apt.scripts plugin
// -> fetch the the values in the constructor of this type
var RUN_CONTINUE= null;
var RUN_DONE= null;

var now= function() {
	return new Date().getTime();
};

dojo.declare("com.ibm.team.apt.client.internal.plancheck.PlanCheckEngine", null, { //$NON-NLS-1$
	
	__checks: null,
	__cancelled: false,

	__queue: null,
	__timeoutToken: null,
	
	__progressLastUpdate: 0,
	__progressLabel: null,
	__progressPercentage: 0,

	constructor: function(plan, reportAcceptFn) {
		RUN_CONTINUE= com.ibm.team.apt.client.plancheck.PlanCheck.Run.CONTINUE;
		RUN_DONE= com.ibm.team.apt.client.plancheck.PlanCheck.Run.DONE;

		this.__plan= plan;
		this.__checks= [];
		this.__queue= [];
		this.__reportAcceptFn= reportAcceptFn;
		this.__workerCallbackFn= dojo.hitch(this, this.__planCheckWorker);
		
		this.__planListener= dojo.hitch(this, function(event) {
			if (!this.__cancelled) {
				this.runDeltaCheck(event.getDelta());
			}
			return undefined;
		});
		
		plan.addListener(this.__planListener);
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	addCheck: function(planCheck) {
		this.__checks.push(planCheck);
	},
	
	getChecks: function() {
		return this.__checks;
	},

	runFullCheck: function() {
		this.__runCheck("getFullCheckElements", []); //$NON-NLS-1$
	},
	
	runDeltaCheck: function(delta) {
		this.__runCheck("getDeltaCheckElements", [ delta ]); //$NON-NLS-1$
	},
	
	runCheck: function(elements) {
		this.__runCheck("getCheckElements", [ elements ]); //$NON-NLS-1$
	},
	
	cancel: function() {
		this.__cancelled= true;
		this.__plan.removeListener(this.__planListener);

		if (this.__timeoutToken) {
			clearTimeout(this.__timeoutToken);
			this.__queue= [];
		}
	},
	
	getProblemDefinition: function(problemId) {
		for (var i= 0; i < this.__checks.length; i++) {
			var result= this.__checks[i].getProblemDefinition(problemId);
			if (result != null)
				return result;
		}
		
		return null;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__runCheck: function(elementComputationFnName, elementComputationArgs) {
		// this scope stores the state of the execution closure we'll return
		
		var planReport= this.__plan.getCheckReport();
		if (planReport == null) {
			planReport= new PlanModelCheckReport(this.__plan);
		}
		
		var checks= this.__checks;
		var checkIndex= 0;
		var check;
		
		var items= null;
		var itemIndex= 0;
		
		var runner= null;
		
		var monitor= new ProgressMonitor(dojo.hitch(this, this.__yieldCheck));
		monitor.beginTask(Messages['planCheck_jobName'], checks.length); //$NON-NLS-1$
		var checkMonitor;
		
		this.__progressLastUpdate= 0;

		this.__triggerRun(dojo.hitch(this, function() {
			
			if ((now() - this.__progressLastUpdate) > 500) {
				var newLabel= monitor.calculateLabel();
				
				if (!dojo.isIE) {
					var newProgress= monitor.calculateProgress();
					if (newLabel != this.__progressLabel || newProgress != this.__progressPercentage) {
						getWorkbench().showMessage(bind(Messages['planCheck_progressLabel'], [newLabel, newProgress])); //$NON-NLS-1$
						
						this.__progressLastUpdate= now();
						this.__progressLabel= newLabel;
						this.__progressPercentage= newProgress;
					}
				} else {
					if (newLabel != this.__progressLabel) {
						getWorkbench().showMessage(newLabel); //$NON-NLS-1$
						this.__progressLastUpdate= now();
						this.__progressLabel= newLabel;
					}
				}
				
			}

			if (runner == null && checkIndex < checks.length) {
				// no runner in progress but still some unfinished checks. 
				// Try grabbing the next runner
				
				check= checks[checkIndex];
				if (items == null) {
					// we haven't calculated the items for this check yet
					items= check[elementComputationFnName].apply(check, elementComputationArgs);
					itemIndex= 0;
					checkMonitor= monitor.child(1, ProgressMonitor.LABEL_APPEND);
					checkMonitor.beginTask(check.getName(), items.length);
				}
				
				if (items != null && itemIndex < items.length) {
					// we haven't checked all items for this checker yet 
					var element= items[itemIndex++];
					if (element instanceof com.ibm.team.apt.client.PlanItem) {
						var elementReport= planReport.getReport(element);
						if (elementReport != null) {
							elementReport.clearProblems(check.getProblemIds());
						}
					}
					
					runner= check.checkElement(element, checkMonitor.child(1, ProgressMonitor.LABEL_NONE));
				} else {
					checkIndex++;
					items= null;
				}
			}

			if (runner != null) {
				try {
					if (runner.execute() != RUN_CONTINUE) {
						var problems= runner.getResult();
						if (dojo.isArray(problems)) {
							dojo.forEach(problems, function(problem) {
								var problemDefinition= check.getProblemDefinition(problem.getId());
								if (problemDefinition != null) {
									problem._updateSeverity(Severity.valueOf(problemDefinition.severity));
								}
								planReport.addProblem(problem);
							});
						}
						runner= null;
					}
				} catch (e) {
					console.log("Stopped plan check '%s' due to an exception.", check.declaredClass); //$NON-NLS-1$
					console.log(e);
				}
			}
			
			if (checkIndex < checks.length) {
				return RUN_CONTINUE;
			} else {
				this.__reportAcceptFn(planReport);
				getWorkbench().hideMessageArea();
				return RUN_DONE;
			}
		}));
	},
	
	__triggerRun: function(check, monitor) {
		this.__queue.push({
			check: check,
			monitor: monitor
		});
		
		if (!this.__timeoutToken) {
			this.__planCheckWorker();
		}
	},
	
	__planCheckWorker: function() {
		if (!this.__planCheckWorkerLock) {
			this.__planCheckWorkerLock= true;
			try {
				this.__timeoutToken= null;
				try {
					this.__startTime= now();
					do {
						if (this.__queue.length == 0)
							return;
						
						if (this.__queue[0].check(this.__queue[0].monitor) === RUN_DONE)
							this.__queue.shift();
						
					} while (!this.__yieldCheck());
				} catch (e) {
					console.log("Stopped plan checks due to an exception."); //$NON-NLS-1$
					console.log(e);
				} finally {
					if (this.__queue.length > 0)
						this.__timeoutToken= setTimeout(this.__workerCallbackFn, 0);
				}
			} finally {
				this.__planCheckWorkerLock= false;
			}
		}
	},

	__yieldCheck: function() {
		return (now() - this.__startTime) > 10;
	},
	
	__sentinel: null // terminates this class definition
});


})();
