/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.foundation.common.ProgressMonitor"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var bind= dojo.string.substitute;
var Messages= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var LABEL_NONE= 1;
var LABEL_APPEND= 2;
	
dojo.declare("com.ibm.team.foundation.common.ProgressMonitor", null, { //$NON-NLS-1$

	__cancelCheckFn: null,
	__yieldCheckFn: null,
	__progress: null,

	constructor: function(yieldCheckFn, cancelCheckFn) {
		this.__cancelCheckFn= cancelCheckFn;
		this.__yieldCheckFn= yieldCheckFn;
		this.__progress= new com.ibm.team.foundation.common.ProgressMonitor._Progress(this);
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	isCanceled: function() {
		return this.__cancelCheckFn && this.__cancelCheckFn(); 
	},
	
	beginTask: function(taskName, ticks) {
		this.__progress.beginTask(taskName, ticks);
	},

	worked: function(ticks) {
		this.__progress.worked(ticks);
	},

	child: function(ticks, flags) {
		return this.__progress.child(ticks);
	},

	shouldYield: function() {
		return this.__yieldCheckFn && this.__yieldCheckFn(); 
	},
	
	// ---- query api ------------------------------------------------------------------------------------------------------
	
	calculateLabel: function() {
		return this.__progress.calculateLabel() || ""; //$NON-NLS-1$
	},

	calculateProgress: function() {
		return Math.round(this.__progress.calculateProgress() * 100);
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.foundation.common.ProgressMonitor.LABEL_NONE= LABEL_NONE;
com.ibm.team.foundation.common.ProgressMonitor.LABEL_APPEND= LABEL_APPEND;

dojo.declare("com.ibm.team.foundation.common.ProgressMonitor._Progress", null, { //$NON-NLS-1$

	__rootMonitor: null,
	
	__name: null,
	__ticks: 1,
	__worked: 0,
	
	__currentChild: null,
	__currentChildTicks: 0,
	__currentChildFlags: 0,

	constructor: function(rootMonitor) {
		this.__rootMonitor= rootMonitor;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	isCanceled: function() {
		return this.__rootMonitor.isCanceled(); 
	},
	
	beginTask: function(taskName, ticks) {
		this.__name= taskName;
		this.__ticks= Math.max(Number(ticks), 1);
	},

	worked: function(ticks) {
		this.__childDone();
		this.__worked+= Number(ticks);
	},

	child: function(ticks, flags) {
		this.__childDone();
		this.__currentChild= new com.ibm.team.foundation.common.ProgressMonitor._Progress(this.__rootMonitor);
		this.__currentChildTicks= Number(ticks);
		this.__currentChildFlags= flags || 0;
		return this.__currentChild;
	},

	shouldYield: function() {
		return this.__rootMonitor.shouldYield(); 
	},
	
	// ---- query api ------------------------------------------------------------------------------------------------------

	calculateLabel: function() {
		if (this.__currentChild ) {
			if ((this.__currentChildFlags & LABEL_APPEND) == LABEL_APPEND) {
				return bind(Messages['progressMonitor_append'], [this.__name, this.__currentChild.calculateLabel()]); //$NON-NLS-1$
			} else if ((this.__currentChildFlags & LABEL_NONE) != LABEL_NONE) {
				return this.__currentChild.calculateLabel();
			}
		}
		return this.__name;
	},

	calculateProgress: function() {
		var worked= this.__worked;
		if (this.__currentChild) {
			worked+= this.__currentChild.calculateProgress() * this.__currentChildTicks;
		}
		
		if (worked < 0) {
			worked= 0;
		} else if (worked > this.__ticks) {
			worked= this.__ticks;
		}

		return worked / this.__ticks;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__childDone: function() {
		if (this.__currentChild) {
			this.__currentChild= null;
			this.__currentChildFlags= 0;
			this.worked(this.__currentChildTicks);
		}
	},
	
	__sentinel: null // terminates this class definition
});


})();
